#include "StdAfx.h"
#include "AnimationWin.h"

#using <mscorlib.dll>

using namespace IDVR;

//sets the first control item of the main IDVR window
void AnimationWin::setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_AnimationEditor)
{
	this->m_SubItem_AnimationEditor = n_SubItem_AnimationEditor;
}

//sets the second control item of the main IDVR window
void AnimationWin::setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_AnimationEditor)
{
	this->m_TbSubItem_AnimationEditor = n_TbSubItem_AnimationEditor;
}

//displays the new calculated values to the status bar
void AnimationWin::actualizeStatusBarDisplay(void)
{
	this->stp_FrameCount->Text = String::Concat(FRAMECOUNTTITLE, this->m_FrameCount.ToString());
	this->stp_LengthInSeconds->Text = String::Concat(LENGHTINSECONDSTITLE, this->m_LengthInSeconds.ToString());
}

//actualize each shader model with the new calculated view and light directions
void AnimationWin::actualizeRenderModels(void)
{
	Array* objectIDs = this->m_RayCaster->Data->getObjectIDs();

	for (int index = 0; index < objectIDs->get_Count(); index++)
	{
		IRenderer *renderer = this->m_RayCaster->Data->getObjectMember(index)->get_RenderModel();

		if (renderer->getShader()->GetType() == __typeof(PhongShadingModelSW))
		{
			PhongShadingModelSW *shader = __try_cast<PhongShadingModelSW*>(renderer->getShader());
			shader->setViewLightParameters(this->m_RayCaster->ImagePlane->get_ViewDirection(), 
										   this->m_RayCaster->ImagePlane->getCurrentLeftUpperPoint(), 
										   this->m_Light);
		}
		else if (renderer->getShader()->GetType() == __typeof(ContourShadingModelSW))
		{
			ContourShadingModelSW *shader = __try_cast<ContourShadingModelSW*>(renderer->getShader());
			shader->set_ViewDirection(this->m_RayCaster->ImagePlane->get_ViewDirection());
		}
		else if (renderer->getShader()->GetType() == __typeof(ToneShadingModelSW))
		{
			ToneShadingModelSW *shader = __try_cast<ToneShadingModelSW*>(renderer->getShader());
			shader->set_LightDirection(this->m_Light->OffsetLightDirection);
		}
	}
}

void AnimationWin::renderAnimation(void)
{
	bool isProgressbarActive = true;

	//calculate the progression step per frame for setting the progress bar
	Decimal frameRenderingStep = (float)(100.0 / this->m_FrameCount), frameRenderingStatus = 0;

	//calcualate the angle per frame in radians
	double rotAngle = Convert::ToDouble(this->nud_RotationAngle->Value / this->nud_FramesPerSec->Value) * (Math::PI / 180.0);

	String *fileName = 0;
	Matrix4 *rotMatrix = 0;
	
	//which rotation orientation has been selected
	switch (this->cob_RotationAxis->SelectedIndex)
	{
	case 0: if (this->rab_PositiveRotation->Checked)
				rotMatrix = Matrix4::CreateRotXMatrix(rotAngle);
			else
				rotMatrix = Matrix4::CreateRotXMatrix(-rotAngle);
			break;	
	case 1: if (this->rab_PositiveRotation->Checked)
				rotMatrix = Matrix4::CreateRotYMatrix(rotAngle);
			else
				rotMatrix = Matrix4::CreateRotYMatrix(-rotAngle);
			break;
	case 2: if (this->rab_PositiveRotation->Checked)
				rotMatrix = Matrix4::CreateRotZMatrix(rotAngle);
			else
				rotMatrix = Matrix4::CreateRotZMatrix(-rotAngle);
			break;

	default: break;
	}

	//no further progress bar for "in frame" calcualations needed
	this->m_RayCaster->setProgressWin(NULL);
	this->m_RayCaster->setGUIHandler(NULL);

	//set up progress bar
	ProgressWin *progressWin = new ProgressWin();

	progressWin->StartPosition = FormStartPosition::CenterScreen;
	progressWin->Title = S"Rendering IDVR Animation";
	if (frameRenderingStep > 1.0)
		progressWin->setProgressbarStepRange(Convert::ToInt32(Math::Round(Convert::ToDouble(frameRenderingStep))));
	progressWin->Show();

	//Iterations for renderering each frame
	for (int i = 1; i <= this->m_FrameCount; i++)
	{
		progressWin->Description = String::Concat(S"Frame number ", i.ToString(), S" of ", this->m_FrameCount.ToString(), S" is being rendered.");

		//set the imageplane rotation
		if (this->rab_WorldSystem->Checked)
			this->m_RayCaster->ImagePlane->RotMatrix = Matrix4::MultiplyMatices(this->m_RayCaster->ImagePlane->RotMatrix, rotMatrix);
		else
			this->m_RayCaster->ImagePlane->modifyRotMatrix(rotMatrix);
		this->m_RayCaster->ImagePlane->transformImagePlane();

		//set the light rotation if light poition is not fixed
		if (!this->chb_LightPosFixed->Checked)
		{
			if (this->rab_WorldSystem->Checked)
				this->m_Light->BaseRotation = Matrix4::MultiplyMatices(this->m_Light->BaseRotation, rotMatrix);
			else
				this->m_Light->modifyBaseRotation(rotMatrix);
			this->m_Light->calculateLightDirection();
		}

		actualizeRenderModels();
		
		//render the frame
		this->m_RayCaster->start_Raycasting();

		if (isProgressbarActive)
		{
			frameRenderingStatus = frameRenderingStatus + frameRenderingStep;
			if (frameRenderingStatus > 1.0f)
			{
				try
				{
					if (!progressWin->isWindowActive())
						throw new Exception();

					progressWin->setProgressbarStep();
					frameRenderingStatus = frameRenderingStatus % 1.0f;
				}
				catch(Exception*)
				{ 
					isProgressbarActive = false; 
					break;
				}
			}
		}
		
		//save the rendered frame to the selected path
		fileName = String::Concat(this->teb_SaveDirectory->Text, i.ToString(), S".tiff");
		DrawModelGLSW::GetBitmap(this->m_RayCaster->getFrameBuffer(), 
								this->m_CanvasWidth, this->m_CanvasHeight, 
								Convert::ToInt32(this->m_RayCaster->ImagePlane->Resolution))->Save(fileName, ImageFormat::Tiff);
	}

	if (isProgressbarActive)
		progressWin->Close();
}
