#include "StdAfx.h"
#include "ArcBall.h" 
#using <mscorlib.dll>
                                     

//Arcball sphere constants:
//Diameter is       100.0f
//Radius is         50.0f
//Radius squared is 2500.0f

using namespace System::Drawing;
using namespace IDVR::Tools;

/*
Projects the specified point on the Arcball sphere surface.
This function calculates the surface coordinates of the corresponding point by projection.
*/
void ArcBall::mapToSphere(PointF n_NewPt, Vector3 *n_NewVec) 
{
    PointF TempPt;
    float length;

    //Copy paramter into temp point
    TempPt = n_NewPt;

    //Adjust point coords and scale down to range of [-1 ... 1]
    TempPt.X  =        (TempPt.X * this->AdjustWidth)  - 1.0f;
    TempPt.Y  = 1.0f - (TempPt.Y * this->AdjustHeight);

    //Compute the square of the length of the vector to the point from the center
    length      = (TempPt.X * TempPt.X) + (TempPt.Y * TempPt.Y);

    //If the point is mapped outside of the sphere... (length > radius squared)
	if (length > RADIUSSQUARED)
    {
        float norm;

        //Compute a normalizing factor (radius / sqrt(length))
		norm    = RADIUS/ (float)Math::Sqrt(length);

        //Return the "normalized" vector, a point on the sphere
		n_NewVec->X = (TempPt.X * norm);
		n_NewVec->Y = (TempPt.Y * norm);
		n_NewVec->Z = 0.0f;
    }
    else    //Else it's on the inside
    {
        //Return a vector to a point mapped inside the sphere sqrt(radius squared - length)
        n_NewVec->X = TempPt.X;
        n_NewVec->Y = TempPt.Y;
		n_NewVec->Z = Math::Sqrt(RADIUSSQUARED - length);
    }
}

/*
Constructor initialization with the valid mouse bounds for canvas height and width.
This valid bound area specifies all valied values for possible mouse point, all point outside this
area will be clipped during the calcualtion.
*/
ArcBall::ArcBall(float NewWidth, float NewHeight)
{
    //Clear initial values
    this->StVec = new Vector3();
    this->EnVec = new Vector3();


    //Set initial bounds
    this->setBounds(NewWidth, NewHeight);
}

//Set new mouse bounds for heigth and width.
void ArcBall::setBounds(float NewWidth, float NewHeight)
{
    //Set adjustment factor for width/height
    this->AdjustWidth  = 1.0f / ((NewWidth  - 1.0f) * 0.5f);
    this->AdjustHeight = 1.0f / ((NewHeight - 1.0f) * 0.5f);
}

//Saves the current specified mouse click point as the new start point.
void ArcBall::click(PointF NewPt)
{
    //Map the point to the sphere
    this->mapToSphere(NewPt, this->StVec);
}

//Saves the current specified mouse drag point as the new end point and calculates the corresponding rotation.
void  ArcBall::drag(PointF NewPt, Vector4 *NewRot)
{
    //Map the point to the sphere
    this->mapToSphere(NewPt, this->EnVec);

    //Return the quaternion equivalent to the rotation
    if (NewRot != NULL)
    {
        //Compute the vector perpendicular to the begin and end vectors
		Vector3 *Perp = Vector3::CrossProd(this->StVec, this->EnVec);

        //Compute the length of the perpendicular vector
		if (Perp->magnitude() > Epsilon)    //if its non-zero
        {
            //We're ok, so return the perpendicular vector as the transform after all
			NewRot->X = Perp->X;
			NewRot->Y = Perp->Y;
            NewRot->Z = Perp->Z;
            //In the quaternion values, w is cosine (theta / 2), where theta is rotation angle
			NewRot->W= Vector3::DotProd(this->StVec, this->EnVec);
        }
        else                                    //if its zero
        {
            //The begin and end vectors coincide, so return an identity transform
            NewRot->X = 0.0f;
            NewRot->Y = 0.0f;
            NewRot->Z = 0.0f;
			NewRot->W = 0.0f;
        }
	}
}

//Resets the current start and end vectors of the rotation calculation.
void ArcBall::resetVectors(void)
{
	this->StVec = new Vector3();
	this->EnVec = new Vector3();
}

//Sets the value of this matrix to the matrix conversion of the quaternion argument. 
//hack this can be optimized some(if s == 0)
void ArcBall::CreateRotationMatrixFromVector4(Matrix4 *n_DestinationObj, Vector4 *n_Vector4)
 {
    double n, s;
    double xs, ys, zs;
	double wx, wy, wz;
    double xx, xy, xz;
    double yy, yz, zz;

    n = (n_Vector4->X * n_Vector4->X) + (n_Vector4->Y * n_Vector4->Y) + (n_Vector4->Z * n_Vector4->Z) + (n_Vector4->W * n_Vector4->W);
    s = (n > 0.0f) ? (2.0f / n) : 0.0f;

    xs = n_Vector4->X * s;  ys = n_Vector4->Y * s;  zs = n_Vector4->Z * s;
    wx = n_Vector4->W * xs; wy = n_Vector4->W * ys; wz = n_Vector4->W * zs;
    xx = n_Vector4->X * xs; xy = n_Vector4->X * ys; xz = n_Vector4->X * zs;
    yy = n_Vector4->Y * ys; yz = n_Vector4->Y * zs; zz = n_Vector4->Z * zs;

	/*n_DestinationObj->setItem(0, 0, (1.0f - (yy + zz))); n_DestinationObj->setItem(1, 0, (xy - wz));          n_DestinationObj->setItem(2, 0, (xz + wy));
    n_DestinationObj->setItem(0, 1, (xy + wz));          n_DestinationObj->setItem(1, 1, (1.0f - (xx + zz))); n_DestinationObj->setItem(2, 1, (yz - wx));
    n_DestinationObj->setItem(0, 2, (xz - wy));          n_DestinationObj->setItem(1, 2, (yz + wx));          n_DestinationObj->setItem(2, 2, (1.0f - (xx + yy)));*/

	n_DestinationObj->setItem(0, 0, (1.0f - (yy + zz))); n_DestinationObj->setItem(0, 1, (xy - wz));          n_DestinationObj->setItem(0, 2, (xz + wy));
    n_DestinationObj->setItem(1, 0, (xy + wz));          n_DestinationObj->setItem(1, 1, (1.0f - (xx + zz))); n_DestinationObj->setItem(1, 2, (yz - wx));
    n_DestinationObj->setItem(2, 0, (xz - wy));          n_DestinationObj->setItem(2, 1, (yz + wx));          n_DestinationObj->setItem(2, 2, (1.0f - (xx + yy)));

	n_DestinationObj->setItem(0,3,0);n_DestinationObj->setItem(1,3,0);n_DestinationObj->setItem(2,3,0);n_DestinationObj->setItem(3,3,1);
	n_DestinationObj->setItem(3,0,0);n_DestinationObj->setItem(3,1,0);n_DestinationObj->setItem(3,2,0);
 }

/*
Sets the rotational component (upper 3x3) of this matrix to the matrix
values in the T precision Matrix3d argument. 
The other elements of this matrix are unchanged; a singular value decomposition is performed
on this object's upper 3x3 matrix to factor out the scale, then this
object's upper 3x3 matrix components are replaced by the passed rotation
components, and then the scale is reapplied to the rotational
components.
*/
 void ArcBall::SetRotationFromMatrix(Matrix4* n_DestinationObj, Matrix4* n_SourceObj)
 {
		float scale = getMatrixSVD(n_DestinationObj);

		setRotationScaleFromMatrix(n_DestinationObj, n_SourceObj);
		setMatrixMulRotationScale(n_DestinationObj, scale);
 }

/*
Calculates a simple SVD value of the specified 4x4 matrix.
This function calculates a very simple SVD value which will be used as a scale factor for actual rotation matrix.
*/
float ArcBall::getMatrixSVD(Matrix4 *n_DestinationObj)
{
        // this is a simple svd.
        // Not complete but fast and reasonable.
        // See comment in Matrix3d.

		float s = (float) Math::Sqrt(
                ( (n_DestinationObj->getItem(0,0) * n_DestinationObj->getItem(0,0)) + (n_DestinationObj->getItem(0,1) * n_DestinationObj->getItem(0,1)) + (n_DestinationObj->getItem(0,2) * n_DestinationObj->getItem(0,2)) + 
                  (n_DestinationObj->getItem(1,0) * n_DestinationObj->getItem(1,0)) + (n_DestinationObj->getItem(1,1) * n_DestinationObj->getItem(1,1)) + (n_DestinationObj->getItem(1,2) * n_DestinationObj->getItem(1,2)) +
                  (n_DestinationObj->getItem(2,0) * n_DestinationObj->getItem(2,0)) + (n_DestinationObj->getItem(2,1) * n_DestinationObj->getItem(2,1)) + (n_DestinationObj->getItem(2,2) * n_DestinationObj->getItem(2,2)) ) / 3.0f );

		return s;
}

//Copies the rotational component (upper 3x3) of the input matrix to a new destination matrix.
void ArcBall::setRotationScaleFromMatrix(Matrix4 *n_DestinationObj, Matrix4* n_SourceObj)
{
        n_DestinationObj->setItem(0,0, n_SourceObj->getItem(0,0)); n_DestinationObj->setItem(0,1, n_SourceObj->getItem(0,1)); n_DestinationObj->setItem(0,2, n_SourceObj->getItem(0,2));
        n_DestinationObj->setItem(1,0, n_SourceObj->getItem(1,0)); n_DestinationObj->setItem(1,1, n_SourceObj->getItem(1,1)); n_DestinationObj->setItem(1,2, n_SourceObj->getItem(1,2));
        n_DestinationObj->setItem(2,0, n_SourceObj->getItem(2,0)); n_DestinationObj->setItem(2,1, n_SourceObj->getItem(2,1)); n_DestinationObj->setItem(2,2, n_SourceObj->getItem(2,2));

		n_DestinationObj->setItem(0,3,0);n_DestinationObj->setItem(1,3,0);n_DestinationObj->setItem(2,3,0);n_DestinationObj->setItem(3,3,1);
		n_DestinationObj->setItem(3,0,0);n_DestinationObj->setItem(3,1,0);n_DestinationObj->setItem(3,2,0);
}

//Multiplies the the rotational component (upper 3x3) of the input matrix with the specified scale value.
void ArcBall::setMatrixMulRotationScale(Matrix4 *n_DestinationObj, float scale)
{
		/*n_DestinationObj->setItem(0,0, n_DestinationObj->getItem(0,0) * scale); n_DestinationObj->setItem(0,1, n_DestinationObj->getItem(0,1) * scale); n_DestinationObj->setItem(0,2, n_DestinationObj->getItem(0,2) * scale); 
		n_DestinationObj->setItem(1,0, n_DestinationObj->getItem(1,0) * scale); n_DestinationObj->setItem(1,1, n_DestinationObj->getItem(1,1) * scale); n_DestinationObj->setItem(1,2, n_DestinationObj->getItem(1,2) * scale); 
		n_DestinationObj->setItem(2,0, n_DestinationObj->getItem(2,0) * scale); n_DestinationObj->setItem(2,1, n_DestinationObj->getItem(2,1) * scale); n_DestinationObj->setItem(2,2, n_DestinationObj->getItem(2,2) * scale); */

		n_DestinationObj->multiply_WithScalar(scale);
		
		n_DestinationObj->setItem(0,3,0);n_DestinationObj->setItem(1,3,0);n_DestinationObj->setItem(2,3,0);n_DestinationObj->setItem(3,3,1);
		n_DestinationObj->setItem(3,0,0);n_DestinationObj->setItem(3,1,0);n_DestinationObj->setItem(3,2,0);
}