#pragma once

using namespace CsGL::OpenGL;
using namespace CsGL::Pointers;
using namespace CsGL::Util;

namespace IDVR {
namespace Tools {
namespace OpenGLTools {

	using namespace System::Drawing;

	/**
	<b>Class for managing the correct deletion of the current canvas.</b>

	This class is part of the OpenGLTools namespace which is actually a subnamespace of the main namespace Tools. 
	<br><br>
	To integrate a correct deletion method in OpenGL to the application's canvas which is implemented in the class
	OpenGLCanvas use only the class.<br>
	As every other class of the subpackage OpenGLTools this class has the base class OpenGLCode of the CSOpenGL library
	to correctly implement the OpenGL code Which will be handled by the OpenGLPanel which will show the specific drawing 
	results on the screen.<br><br>
	The main task of this class is the deletion of the current canvas and reset it to the background color.

	@see OpenGLCanvas - the GUI class for the canvas with its draw control.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005
	*/
public __gc class ClearCanvasGL : public OpenGLPanel::OpenGLCode
{
public:

	/**
	This is the standard constructor.
	All member variables of this new instance will be set to zero/null.
	*/
	ClearCanvasGL(void);

	/**
	This function overwrites the base init funtion of the class OpenGLCode.
	It contains the actual initialization code in OpenGL and also performs the code.
	*/
	virtual void Init(void);
	/**
	This function overwrites the base draw funtion of the class OpenGLCode.
	It contains the actual drawing code in OpenGL and also performs the code.
	*/
	virtual void Draw(void);

	/**
	Sets a new background color for the canvas.
	@param n_CanvasBackColor the new canvas background color.
	*/
	__property void set_CanvasBackColor(Color n_CanvasBackColor);

private:
	float m_CanvasBackColor __gc[]; /**< Saves the current background color of the canvas. */
};
}
}
}