#include "StdAfx.h"
#include ".\ContourShadingModelSW.h"
#using <mscorlib.dll>

#include "DataLoader.h"

using namespace IDVR::Volume;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

//Standard constructor.
ContourShadingModelSW::ContourShadingModelSW(void)
{
	this->m_DensityScope = DataLoader::DensityScope;
	this->m_GradientMagnitudeThreshold = DataLoader::Default_ContourGradientThreshold;
	this->m_Exponent = DataLoader::Default_ContourExponent;
	this->m_BaseColor = new float __gc[3];

	this->m_TransferFunc = Transferfunction::CreateGeneralTransfer(this->m_DensityScope);
	this->m_KTF = DataLoader::Default_ToneKTF;
}

//Constructor.
ContourShadingModelSW::ContourShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_Exponent, Color n_Col)
{
	this->m_DensityScope = n_DensityScope;
	this->m_GradientMagnitudeThreshold = n_GradientMagnitudeThreshold;
	this->m_Exponent = n_Exponent;
	this->m_BaseColor = new float __gc[3];
	this->m_BaseColor[RED] = (float)n_Col.get_R() / 255.0f;
	this->m_BaseColor[GREEN] = (float)n_Col.get_G() / 255.0f;
	this->m_BaseColor[BLUE] = (float)n_Col.get_B() / 255.0f;

	this->m_TransferFunc = NULL;
	this->m_KTF = NULL;
}

//Constructor.
ContourShadingModelSW::ContourShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_Exponent, Color n_Col, float n_KTF, Transferfunction* n_TransferFunc)
{
	this->m_DensityScope = n_DensityScope;
	this->m_GradientMagnitudeThreshold = n_GradientMagnitudeThreshold;
	this->m_Exponent = n_Exponent;
	this->m_BaseColor = new float __gc[3];
	this->m_BaseColor[RED] = (float)n_Col.get_R() / 255.0f;
	this->m_BaseColor[GREEN] = (float)n_Col.get_G() / 255.0f;
	this->m_BaseColor[BLUE] = (float)n_Col.get_B() / 255.0f;

	this->m_TransferFunc = n_TransferFunc;
	this->m_KTF = n_KTF;
}

//Calculates the Color value for a given raysample.
RaySample* ContourShadingModelSW::doShading(RaySample* n_RaySample)
{
	float temp_Color __gc[] = new float __gc[3];
	double temp_Value;

	if(n_RaySample->get_Gradient()->get_ScaledLength() >= this->m_GradientMagnitudeThreshold)
	{
		temp_Value = Math::Pow(1 - Math::Abs(n_RaySample->get_Gradient()->get_GradientVector()->dotProd(this->m_View)), this->m_Exponent);
		temp_Color[RED] = (float)temp_Value * this->m_BaseColor[RED];
		temp_Color[GREEN] = (float)temp_Value * this->m_BaseColor[GREEN];
		temp_Color[BLUE] = (float)temp_Value * this->m_BaseColor[BLUE];

		n_RaySample->set_Opacity((float)temp_Value);
	}
	else
	{
		temp_Color[RED] = temp_Color[GREEN] = temp_Color[BLUE] = 0;
		n_RaySample->set_Opacity(0);
	}

	//no transfer funtion is specified => LMip - compositing
	if(this->m_TransferFunc != NULL)
	{	
		/*
		temp_Color[RED] = (temp_Color[RED] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[RED])) / 2;
		temp_Color[GREEN] = (temp_Color[GREEN] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[GREEN])) / 2;
		temp_Color[BLUE] = (temp_Color[BLUE] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[BLUE])) / 2;
		*/

		//blends according to: contour color : transfer function color = 1 - ktf : ktf
		temp_Color[RED] = ((1 - this->m_KTF) * temp_Color[RED]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[RED]);
		temp_Color[GREEN] = ((1 - this->m_KTF) * temp_Color[GREEN]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[GREEN]);
		temp_Color[BLUE] = ((1 - this->m_KTF) * temp_Color[BLUE]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[BLUE]);

		n_RaySample->set_Opacity(this->m_TransferFunc->getOpacity(n_RaySample->get_Density()));
	}

	n_RaySample->set_Color(temp_Color);
	
	return n_RaySample;
}

//Sets the view direction.
void ContourShadingModelSW::set_ViewDirection(Vector3* n_View)
{
	this->m_View = n_View;
}

//Returns the gradient magnitude threshold.
double ContourShadingModelSW::get_ContourGradientMagnitudeThreshold(void)
{
	return this->m_GradientMagnitudeThreshold;
}

//Returns the exponent.
double ContourShadingModelSW::get_Exponent(void)
{
	return this->m_Exponent;
}

//Returns the current color as Color object.
Color ContourShadingModelSW::get_Col(void)
{
	return Color::FromArgb(Convert::ToInt32(Math::Round(this->m_BaseColor[RED] * 255.0)), 
		                   Convert::ToInt32(Math::Round(this->m_BaseColor[GREEN] * 255.0)), 
						   Convert::ToInt32(Math::Round(this->m_BaseColor[BLUE] * 255.0)));
}

//Returns the ktf value.
float ContourShadingModelSW::get_ContourKTF(void)
{
	return this->m_KTF;
}