//MipShader with LMIP setting

//an ALPHA_FLOAT16_ATI texture with all density values of the volume dataset
uniform sampler3D density_tex;

//an RGBA_FLOAT16_ATI texture contains the entry point for each ray and the number of Samples along the ray
uniform sampler2D entryPoint_tex;

//an GL_RGBA8 texture contains the current composited color and opacityvalues along the ray
uniform sampler2D globalBuffer_tex;

//an GL_RGBA8 texture contains the current transfer function
uniform sampler1D transferFunction_tex;

//contains the current background color
uniform vec3 backColor;

//contains the offset from the entry point to the current ray sample coordinates
uniform vec3 rayOffset;

//the current number of the ray sample
uniform float currentRaySampleNumber;

//contains the current threshold for opacity and gradient magnitude
uniform struct {
  float opacityThreshold;
  float gradientThreshold;
} thresholds;

const float OUTOFVOLUME = 0.0;

//the actual calculation for a specific fragment
void main()
{
	//early ray termination
	if(thresholds.opacityThreshold <= texture2D(globalBuffer_tex, gl_FragCoord.xy).a)
	{
		gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
		gl_FragDepth = gl_FragCoord.z - 1.0;
		return;
	}

	//calculate the current ray sample coordinates
	vec4 entryPoint = texture2D(entryPoint_tex, gl_FragCoord.xy);

	//if fragment is no part of the volume set
	if(entryPoint.a == OUTOFVOLUME)
	{
		gl_FragColor = vec4(backColor, 1.0);
		gl_FragDepth = gl_FragCoord.z - 1.0;
		return;
	}
		
	vec3 raySampleCoords = vec3(entryPoint) + rayOffset;

	vec4 fragDensity = texture3D(density_tex, raySampleCoords);

	//get the color and opacity values from TF
	gl_FragColor = texture1D(transferFunction_tex, fragDensity.x);
	
	//check whether the end of the current ray has been reached
	/* if(entryPoint.a <= currentRaySampleNumber)
		gl_FragDepth = gl_FragCoord.z - 1;
	else
		gl_FragDepth = gl_FragCoord.z + 1; */
  
  gl_FragDepth = (entryPoint.a <= currentRaySampleNumber) ? gl_FragCoord.z - 1.0 : gl_FragCoord.z + 1.0;
	
}