#include "StdAfx.h"
#include ".\GLSLShaderDataManager.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::HWVolumeRenderer;
using namespace System::Collections;

//Constructor to initalize the instace variables
GLSLShaderDataManager::GLSLShaderDataManager(void)
{
	this->m_Density_tex = 0;
	this->m_EntryPoint_tex = 0;
	this->m_TransferFunction_tex = 0;
	this->m_ObjectID_tex = 0;

	this->m_MaxNumOfSamples = 0;
}

//creates a ready-to-use 3D texture object with all density values of the current data set
void GLSLShaderDataManager::createDensityTextureObject(VolumeData *n_Dataset, unsigned char n_InterpolationMode)
{
	unsigned short volumeDimension __gc[] = n_Dataset->VolumeDimension;
	GLhalf *densities = new GLhalf[n_Dataset->VolumeDimension[X], n_Dataset->VolumeDimension[Y], n_Dataset->VolumeDimension[Z]];

	for (int depth = 0; depth < n_Dataset->VolumeDimension[Z]; depth++)
		for (int row = 0; row < n_Dataset->VolumeDimension[Y]; row++)
			for (int column = 0; column < n_Dataset->VolumeDimension[X]; column++)
				densities[column, row, depth] = n_Dataset->VoxelDataset->getVoxelDensity(column, row, depth);

	glBindTexture(GL_TEXTURE_3D, this->m_Density_tex);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_R, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	if (n_InterpolationMode == TRILINEAR)
	{
		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}
	else
	{
		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	}

	glTexImage3D(GL_TEXTURE_3D, 0, GL_INTENSITY_FLOAT16_ATI, n_Dataset->VolumeDimension[X], n_Dataset->VolumeDimension[Y],
		n_Dataset->VolumeDimension[Z], 0, GL_INTENSITY_FLOAT16_ATI, GL_HALF_FLOAT_NV, densities);

	delete densities;
}

//creates a ready-to-use 3D texture object with all ObjectID values of the current data set
void GLSLShaderDataManager::createObjectIDTextureObject(VolumeData *n_Dataset)
{
	unsigned short volumeDimension __gc[] = n_Dataset->VolumeDimension;
	GLubyte *objectIDs = new GLubyte[n_Dataset->VolumeDimension[X], n_Dataset->VolumeDimension[Y], n_Dataset->VolumeDimension[Z]];

	for (int depth = 0; depth < n_Dataset->VolumeDimension[Z]; depth++)
		for (int row = 0; row < n_Dataset->VolumeDimension[Y]; row++)
			for (int column = 0; column < n_Dataset->VolumeDimension[X]; column++)
				objectIDs[column, row, depth] = (unsigned char)n_Dataset->VoxelDataset->getVoxelObjectID(column, row, depth);

	glBindTexture(GL_TEXTURE_3D, this->m_ObjectID_tex);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_R, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glTexImage3D(GL_TEXTURE_3D, 0, GL_INTENSITY8, n_Dataset->VolumeDimension[X], n_Dataset->VolumeDimension[Y],
		n_Dataset->VolumeDimension[Z], 0, GL_INTENSITY8, GL_UNSIGNED_BYTE, objectIDs);

	delete objectIDs;
}

//creates a ready-to-use 2D texture object containing the current transfer function from data set
void GLSLShaderDataManager::createTransferFunctionTextureObjectFromDataset(Transferfunction *n_Tranferfunction, unsigned short n_ObjectID)
{
	GLfloat *transferFunction = new GLfloat[n_Tranferfunction->FunctionLength, 4];
	float tempColor __gc[];

	for (int index = 0; index < n_Tranferfunction->FunctionLength; index++)
	{
		tempColor = n_Tranferfunction->getColor(index);
		transferFunction[index, R] = tempColor[R];
		transferFunction[index, G] = tempColor[G];
		transferFunction[index, B] = tempColor[B];
		transferFunction[index, A] = n_Tranferfunction->getOpacity(index);
	}
		
	glBindTexture(GL_TEXTURE_1D, this->m_TransferFunction_tex);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage1D(GL_TEXTURE_1D, 0, GL_RGBA, n_Tranferfunction->FunctionLength, 0, GL_RGBA, GL_FLOAT, transferFunction);

	delete transferFunction;
}

//creates a ready-to-use 2D texture object containing the the standard linar transfer function
void GLSLShaderDataManager::createTransferFunctionTextureObjectFromInterpolation(Color n_Color, unsigned short n_TransferfunctionLength)
{
	GLfloat *transferFunction = new GLfloat[n_TransferfunctionLength, 4];
	float beginColor __gc[] = {0.0f,0.0f,0.0f},
		  endColor __gc[] = {(float)n_Color.get_R() / 255.0f, (float)n_Color.get_G() / 255.0f, (float)n_Color.get_B() / 255.0f};
	float currentWeight = 0.0f;


	for (int index = 0; index < n_TransferfunctionLength; index++)
	{
		currentWeight = (float)index / n_TransferfunctionLength;

		transferFunction[index, R] = (1.0f - currentWeight) * beginColor[R] + currentWeight * endColor[R];
		transferFunction[index, G] = (1.0f - currentWeight) * beginColor[G] + currentWeight * endColor[G];
		transferFunction[index, B] = (1.0f - currentWeight) * beginColor[B] + currentWeight * endColor[B];
		transferFunction[index, A] = currentWeight;
	}
		
	glBindTexture(GL_TEXTURE_1D, this->m_TransferFunction_tex);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage1D(GL_TEXTURE_1D, 0, GL_RGBA, n_TransferfunctionLength, 0, GL_RGBA, GL_FLOAT, transferFunction);

	delete transferFunction;
}

//creates a ready-to-use 2D texture object containing the entry point coords depending on the current view direction
void GLSLShaderDataManager::createEntryPointTextureObject(VolumeData *n_Dataset, ImagePlane *n_ImagePlane, float n_RaySampleDistance)
{
	GLfloat *entryPoints = new GLfloat[n_ImagePlane->Resolution, n_ImagePlane->Resolution, 4];
	float tempNumOfSamples = 0, tempMaxNumOfSamples = 0;
	Vector3 *nextStartPos, *intersectionPoints __gc[]; 

	for (int row = 0; row < n_ImagePlane->Resolution; row++)
		for (int column = 0; column < n_ImagePlane->Resolution; column++)
		{
			nextStartPos = Vector3::Addition(n_ImagePlane->getCurrentLeftBottomPoint(), n_ImagePlane->getHorizontalViewVector()->multSkalar((double)column + 0.5));
			nextStartPos = Vector3::Addition(nextStartPos, n_ImagePlane->getVerticalViewVector()->multSkalar((double)row + 0.5));
			intersectionPoints = calculate_IntersectionPoints(nextStartPos, n_ImagePlane->ViewDirection, n_Dataset->VolumeDimension);
			
			if(intersectionPoints[0] == NULL && intersectionPoints[1] == NULL)
			{
				entryPoints[column, row, R] = 0.0f;
				entryPoints[column, row, G] = 0.0f;
				entryPoints[column, row, B] = 0.0f;
				entryPoints[column, row, A] = 0.0f;
				continue;
			}

			//calculate the maximum count of sample point calculations
			tempNumOfSamples = Convert::ToSingle(Math::Floor(Vector3::Subtraction(intersectionPoints[1], intersectionPoints[0])->magnitude() / n_RaySampleDistance));
			tempMaxNumOfSamples = Math::Max(tempMaxNumOfSamples, tempNumOfSamples);

			entryPoints[column, row, R] = (float)intersectionPoints[0]->X;
			entryPoints[column, row, G] = (float)intersectionPoints[0]->Y;
			entryPoints[column, row, B] = (float)intersectionPoints[0]->Z;
			entryPoints[column, row, A] = tempNumOfSamples;
		}

	glBindTexture(GL_TEXTURE_2D, this->m_EntryPoint_tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, (int)n_ImagePlane->Resolution, (int)n_ImagePlane->Resolution, 0, GL_RGBA, GL_FLOAT, entryPoints);

	this->m_MaxNumOfSamples = (int)tempMaxNumOfSamples;
	delete entryPoints;
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, int n_VariableData)
{
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform1iARB(location, n_VariableData);
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, float n_VariableData)
{
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform1fARB(location, n_VariableData);
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, bool n_VariableData)
{
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform1iARB(location, n_VariableData);
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, Vector2 *n_VariableData)
{
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform2fARB(location, (float)n_VariableData->X, (float)n_VariableData->Y);
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, Vector3 *n_VariableData)
{
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform3fARB(location, (float)n_VariableData->X, (float)n_VariableData->Y, (float)n_VariableData->Z);
}

//sets a specific uniform variable of the current activated fragment shader (program object)
void GLSLShaderDataManager::setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, GLuint n_TextureName, GLint n_TextureUnit, GLuint n_Texturetyp)
{
	//activate specifiv texture unit and bind with texture object
	glActiveTexture(GL_TEXTURE0 + n_TextureUnit);
	glBindTexture(n_Texturetyp, n_TextureName);
	
	//get location in program object and set the current texture unit
	GLint location = glGetUniformLocationARB(n_ProgramObject, n_VariableName);
	glUniform1iARB(location, n_TextureUnit);
}

//sets the given fragment object to OpenGL state
void GLSLShaderDataManager::activateProgramObject(GLhandleARB n_ProgramObject)
{
	glUseProgramObjectARB(n_ProgramObject);
}

//resets the OpenGL state to foxed functionality
void GLSLShaderDataManager::deactivateProgramObject(GLhandleARB n_ProgramObject)
{
	glUseProgramObjectARB(0);
}

GLuint GLSLShaderDataManager::get_DensityTexture(void)
{
	return this->m_Density_tex;
}

GLuint GLSLShaderDataManager::get_ObjectIDTexture(void)
{
	return this->m_ObjectID_tex;
}

GLuint GLSLShaderDataManager::get_TransferFunctionTexture(void)
{
	return this->m_TransferFunction_tex;
}

GLuint GLSLShaderDataManager::get_EntryPointTexture(void)
{
	return this->m_EntryPoint_tex;
}

Vector3* GLSLShaderDataManager::calculate_IntersectionPoints(Vector3* n_StartPoint, Vector3 *n_ViewDirection, unsigned short n_VolumeDimension __gc[]) __gc[]
{
	bool haveIntersectionPoint = true;
	ArrayList* intersectionPoints = new ArrayList(6);
	Vector3* intersectionPointsCoords __gc[] = new Vector3*[Z]; 
	double A = 0.0, B = 0.0, C = 0.0, D = 0.0, lambda = -1.0;
	
	//Left plane
	A = 1.0, B = 0.0, C = 0.0, D = -1.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
				 (A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->Y >= 1.0 && intersection->Y <= n_VolumeDimension[Y] - 2) && (intersection->Z >= 1.0 && intersection->Z <= n_VolumeDimension[Z] - 2))
			intersectionPoints->Add(intersection);
	}
	
	//Right plane
	A = 1.0, B = 0.0, C = 0.0, D = -n_VolumeDimension[X] + 2.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
				 (A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->Y >= 1.0 && intersection->Y <= n_VolumeDimension[Y] - 2) && (intersection->Z >= 1.0 && intersection->Z <= n_VolumeDimension[Z] - 2))
			intersectionPoints->Add(intersection);
	}

	//Top plane
	A = 0.0, B = 1.0, C = 0.0, D = -1.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
				 (A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= n_VolumeDimension[X] - 2) && (intersection->Z >= 1.0 && intersection->Z <= n_VolumeDimension[Z] - 2))
			intersectionPoints->Add(intersection);
	}

	//Bottom plane
	A = 0.0, B = 1.0, C = 0.0, D = -n_VolumeDimension[Y] + 2.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
				 (A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= n_VolumeDimension[X] - 2) && (intersection->Z >= 1.0 && intersection->Z <= n_VolumeDimension[Z] - 2))
			intersectionPoints->Add(intersection);
	}

	//Front plane
	A = 0.0, B = 0.0, C = 1.0, D = -1.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
				 (A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= n_VolumeDimension[X] - 2) && (intersection->Y >= 1.0 && intersection->Y <= n_VolumeDimension[Y] - 2))
			intersectionPoints->Add(intersection);
	}

	//Back plan
	A = 0.0, B = 0.0, C = 1.0, D = -n_VolumeDimension[Z] + 2.0;

	if((A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z) != 0)
	{
		lambda = (-(A * n_StartPoint->X + B * n_StartPoint->Y + C * n_StartPoint->Z + D)) /
			(A * n_ViewDirection->X + B * n_ViewDirection->Y + C * n_ViewDirection->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(n_StartPoint, n_ViewDirection->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= n_VolumeDimension[X] - 2) && (intersection->Y >= 1.0 && intersection->Y <= n_VolumeDimension[Y] - 2))
			intersectionPoints->Add(intersection);
	}

	//Check which is entry point and exit point
	if(intersectionPoints->Count > 1)
	{
		double distance __gc[] = new double __gc[intersectionPoints->Count];

		double maximalDistance = Double::MinValue;
		double minimalDistance = Double::MaxValue;
		int minimalIndex = -1;
		int maximalIndex = -1;
		
		for(int elements = 0; elements < distance->get_Count(); elements++)
		{
			distance[elements] = Vector3::Subtraction(__try_cast<Vector3*>(intersectionPoints->get_Item(elements)), n_StartPoint)->magnitude();

			if(distance[elements] < minimalDistance)
			{
				minimalDistance = distance[elements];
				minimalIndex = elements;
			}
			if(distance[elements] > maximalDistance)
			{
				maximalDistance = distance[elements];
				maximalIndex = elements;
			}
		}

		intersectionPointsCoords[X] =  __try_cast<Vector3*>(intersectionPoints->get_Item(minimalIndex));
		intersectionPointsCoords[Y] =  __try_cast<Vector3*>(intersectionPoints->get_Item(maximalIndex));
	}
	else
	{
		intersectionPointsCoords[X] = intersectionPointsCoords[Y] = NULL;
	}

	intersectionPoints = 0;
	return intersectionPointsCoords;
}
