#include "StdAfx.h"
#include ".\gradient.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;

/*
This is the standard constructor.
All elements of the gradient vector will be set to zero.
*/
Gradient::Gradient(void)
{
	this->m_GradientVector = new Vector3();
	this->m_Length = 0;
}

/*
Constructor initialization by a specified instance of the Vector3 class.
This constructor initialized all gradient elements with the corresponding elements of the Vector3 instance. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length.
*/
Gradient::Gradient(Vector3* n_Vector3)
{
	this->m_Length = n_Vector3->magnitude();
	this->m_GradientVector = n_Vector3;
}

/*
Constructor initialization by a specified instance of the Vector3 class.
This constructor initialized all gradient elements with the corresponding elements of the Vector3 instance. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length. Additionally,
a user-defined scale factor will be used instead of the basically defined scale factor of this class.
*/
Gradient::Gradient(Vector3* n_Vector3, double n_ScaleFactor)
{
	this->m_Length = n_Vector3->magnitude();
	this->m_GradientVector = n_Vector3;
	this->m_ScaleFactor = n_ScaleFactor;
}

/*
Constructor initialization by three specified double parameters.
This constructor initialized all gradient elements with the corresponding elements of the double parameters. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length.
*/
Gradient::Gradient(double n_X, double n_Y, double n_Z)
{
	Vector3* temp = new Vector3(n_X, n_Y, n_Z);
	this->m_Length = temp->magnitude();
	this->m_GradientVector = temp;
}

/*
Constructor initialization by three specified double parameters.
This constructor initialized all gradient elements with the corresponding elements of the double parameters. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length. Additionally,
a user-defined scale factor will be used instead of the basically defined scale factor of this class.
*/
Gradient::Gradient(double n_X, double n_Y, double n_Z, double n_ScaleFactor)
{
	Vector3* temp = new Vector3(n_X, n_Y, n_Z);
	this->m_Length = temp->magnitude();
	this->m_GradientVector = temp;
	this->m_ScaleFactor = n_ScaleFactor;
}

/*
Constructor initialization by an array of the gradient vector elements.
This constructor initialized all gradient elements with the corresponding elements of the double element array. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length.
*/
Gradient::Gradient(float n_NewValues __gc[])
{
	Vector3* temp = new Vector3(n_NewValues[0], n_NewValues[1], n_NewValues[2]);
	this->m_Length = temp->magnitude();
	this->m_GradientVector = temp;
}

/*
Constructor initialization by an array of the gradient vector elements.
This constructor initialized all gradient elements with the corresponding elements of the double element array. The gradient
length will be automatically normalized to unit length and the original length will be saved in Gradient::m_Length. Additionally,
a user-defined scale factor will be used instead of the basically defined scale factor of this class.
*/
Gradient::Gradient(float n_NewValues __gc[], double n_ScaleFactor)
{
	Vector3* temp = new Vector3(n_NewValues[0], n_NewValues[1], n_NewValues[2]);
	this->m_Length = temp->magnitude();
	this->m_GradientVector = temp;
	this->m_ScaleFactor = n_ScaleFactor;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Returns the original length of the gradient vector.
double Gradient::get_Length(void)
{
	return this->m_Length;
}

/*
Returns the scaled length of the gradient vector.
The actual scaling mode deponds on the currently used scale factor Gradient::m_ScaleFactor.
*/
double Gradient::get_ScaledLength(void)
{
	return this->m_ScaledLength;
}

//Returns the gradient vector as an instance of the Vector3 class.
Vector3* Gradient::get_GradientVector(void)
{
	return this->m_GradientVector;
}

//Sets the gradient vector by an instance of the Vector3 class.
void Gradient::set_GradientVector(Vector3* n_Vector3)
{
	this->m_GradientVector = n_Vector3;
}

//Sets the scale factor by an arbitrary user-defineed value.
void Gradient::set_ScaleFactor(double n_ScaleFactor)
{
	this->m_ScaleFactor = n_ScaleFactor;
}

/*
Calculates the scaled length with the current scale factor.
The calculation of the scaled length will be applied by dividing the gradient length by the scale factor Gradient::m_ScaleFactor.
*/
void Gradient::scale_Length(void)
{
	this->m_ScaledLength = this->m_Length / this->m_ScaleFactor;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Creates a normalized gradient instance by the specified gradient vector.
Gradient* Gradient::CreateNormalizedGradient(Gradient* n_Gradient)
{
	n_Gradient->set_GradientVector(Vector3::CreateNormalizedVector(n_Gradient->get_GradientVector()));
	return n_Gradient;
}