#pragma once

#include ".\IGradientEstimation.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace RenderCalculations {

using namespace System;
using namespace IDVR::Interfaces;
using namespace IDVR::Volume::DataStructures;

/**
<B>Class for the gradient estimation.</B>

namespace: IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations

Baseclass: IDVR::Interfaces::IGradientEstimation

This class provides methods to estimate the gradient either by central differences, or by 4d linear regression.

@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-19-2005
*/

public __gc class GradientEstimationSW : public IGradientEstimation
{

private:

	double m_scaleFactorDifferences; /**< Maximum possible value of the gradient by central differences. */
	double m_scaleFactorRegression; /**< Maximum possible value of the gradient by 4d linear regression */

	__value enum Indices{X = 0, Y, Z};

public:

	/**
	Standard constuctor.
	*/
	GradientEstimationSW(void);

	/**
	Calculates the gradient by central differendes.
	@param n_Column column of the gradient.
	@param n_Row row of the gradient.
	@param n_Depth depth of the gradient.
	@param n_DataSet the dataset.
	@return the estimated gradient.
	*/
	virtual float calculate_NormalEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[];
	/**
	Calculates the gradient by 4d inear regression.
	@param n_Column column of the gradient.
	@param n_Row row of the gradient.
	@param n_Depth depth of the gradient.
	@param n_DataSet the dataset.
	@return the estimated gradient.
	*/
	virtual float calculate_4DLinerRegressionEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[];

	/**
	Returns the scale factor of central differences.
	@return The scale factor of central differences.
	*/
	__property double get_scaleFactorDifferences(void);
	/**
	Returns the scale factor of 4d linear regression.
	@return The scale factor of 4d linear regression.
	*/
	__property double get_scaleFactorRegression(void);

	static const char BY_DIFFERENCES = 0; /**< gradient estimation by central differences. */
	static const char BY_4DREGRESSION = 1; /**< gradient estimation by 4d linear regression. */
};
}
}
}
}


