#include "StdAfx.h"
#include ".\GradientInterpolationWithoutPreCalcSW.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;

//Standard constructor.
GradientInterpolationWithoutPreCalcSW::GradientInterpolationWithoutPreCalcSW(void)
{
}

//Interpolates the gradient of a given raysample.
Gradient* GradientInterpolationWithoutPreCalcSW::interpolateGradient(unsigned char intMode, unsigned char grMode, RaySample* n_RaySample, VolumeData* n_DataSet, 
											double x_down, double x_up, double y_down, double y_up, double z_down, double z_up)
{
	Gradient* temp_Gradient;
	GradientEstimationSW* calcGradient = new GradientEstimationSW();

	unsigned short sh_x_down = (unsigned short)x_down,
				   sh_x_up =   (unsigned short)x_up,
				   sh_y_down = (unsigned short)y_down,
				   sh_y_up =   (unsigned short)y_up,
				   sh_z_down = (unsigned short)z_down,
				   sh_z_up =   (unsigned short)z_up;

	//Gradient interpolation for the specific sample by TriLinear interpolation
	if (intMode == GradientInterpolationWithoutPreCalcSW::BY_TRILINIEAR)
	{
		 double x = n_RaySample->get_Coords()->X;
		 double y = n_RaySample->get_Coords()->Y;
		 double z = n_RaySample->get_Coords()->Z;

		 //get the calculated gradient values of neighbour voxels
		 Gradient* v_000 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_down, sh_z_down));
		 Gradient* v_100 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up,   sh_y_down, sh_z_down));
		 Gradient* v_010 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_up,   sh_z_down));
		 Gradient* v_110 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up,   sh_y_up,   sh_z_down));
		 Gradient* v_001 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_down, sh_z_up));
		 Gradient* v_101 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up,   sh_y_down, sh_z_up));
		 Gradient* v_011 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_up,   sh_z_up));
		 Gradient* v_111 = new Gradient(n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up,   sh_y_up,   sh_z_up));

		 double tempValue1 __gc[] = {(x_up - x), (y_up - y), (z_up - z)};
		 double tempValue2 __gc[] = {(x - x_down), (y - y_down), (z - z_down)};

		 //do trilinear interpolation
		 double temp_X = (v_000->get_GradientVector()->X * tempValue1[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_100->get_GradientVector()->X * tempValue2[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_010->get_GradientVector()->X * tempValue1[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_110->get_GradientVector()->X * tempValue2[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_001->get_GradientVector()->X * tempValue1[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_101->get_GradientVector()->X * tempValue2[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_011->get_GradientVector()->X * tempValue1[X] * tempValue2[Y] * tempValue2[Z]) +
						 (v_111->get_GradientVector()->X * tempValue2[X] * tempValue2[Y] * tempValue2[Z]);

		 double temp_Y = (v_000->get_GradientVector()->Y * tempValue1[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_100->get_GradientVector()->Y * tempValue2[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_010->get_GradientVector()->Y * tempValue1[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_110->get_GradientVector()->Y * tempValue2[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_001->get_GradientVector()->Y * tempValue1[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_101->get_GradientVector()->Y * tempValue2[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_011->get_GradientVector()->Y * tempValue1[X] * tempValue2[Y] * tempValue2[Z]) +
						 (v_111->get_GradientVector()->Y * tempValue2[X] * tempValue2[Y] * tempValue2[Z]);

		 double temp_Z = (v_000->get_GradientVector()->Z * tempValue1[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_100->get_GradientVector()->Z * tempValue2[X] * tempValue1[Y] * tempValue1[Z]) +
						 (v_010->get_GradientVector()->Z * tempValue1[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_110->get_GradientVector()->Z * tempValue2[X] * tempValue2[Y] * tempValue1[Z]) +
						 (v_001->get_GradientVector()->Z * tempValue1[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_101->get_GradientVector()->Z * tempValue2[X] * tempValue1[Y] * tempValue2[Z]) +
						 (v_011->get_GradientVector()->Z * tempValue1[X] * tempValue2[Y] * tempValue2[Z]) +
						 (v_111->get_GradientVector()->Z * tempValue2[X] * tempValue2[Y] * tempValue2[Z]);

		 if (grMode == GradientEstimationSW::BY_DIFFERENCES)
			temp_Gradient = Gradient::CreateNormalizedGradient(new Gradient(temp_X, temp_Y, temp_Z, calcGradient->scaleFactorDifferences));
		 else if (grMode == GradientEstimationSW::BY_4DREGRESSION)
			temp_Gradient = Gradient::CreateNormalizedGradient(new Gradient(temp_X, temp_Y, temp_Z, calcGradient->scaleFactorRegression));
		 temp_Gradient->scale_Length();
	}
	//Gradient interpolation for the specific sample by nearest neighbour interpolation
	else if (intMode == GradientInterpolationWithoutPreCalcSW::BY_NEARESTNEIGHBOUR)
	{
		//Selection of sample`s objectID by nearest neighbour choice
		double temp_d __gc[] = new double __gc[8], distance = Double::MaxValue; int selectedIndex = 0;
		temp_d[0] = Vector3::Subtraction(new Vector3(x_down, y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[1] = Vector3::Subtraction(new Vector3(x_up,   y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[2] = Vector3::Subtraction(new Vector3(x_down, y_up,   z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[3] = Vector3::Subtraction(new Vector3(x_up,   y_up,   z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[4] = Vector3::Subtraction(new Vector3(x_down, y_down, z_up),   n_RaySample->get_Coords())->magnitude();
		temp_d[5] = Vector3::Subtraction(new Vector3(x_up,   y_down, z_up),   n_RaySample->get_Coords())->magnitude();
		temp_d[6] = Vector3::Subtraction(new Vector3(x_down, y_up,   z_up),   n_RaySample->get_Coords())->magnitude();
		temp_d[7] = Vector3::Subtraction(new Vector3(x_up,   y_up,   z_up),   n_RaySample->get_Coords())->magnitude();

		//get the index width the smallest distance
		for (int index = 0; index < 8; index++)
			if (temp_d[index] < distance) 
			{
				distance = temp_d[index]; 
				selectedIndex = index;
			}

		float newGradientValues __gc[];

	    //get the calculated gradient value of the nearest neighbour voxel
		switch(selectedIndex)
		{
		case 0:	newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_down, sh_z_down); break;
		case 1: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up, sh_y_down, sh_z_down);	break;
		case 2: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_up, sh_z_down);	break;
		case 3: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up, sh_y_up, sh_z_down);		break;
		case 4: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_down, sh_z_up);	break;
		case 5: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up, sh_y_down, sh_z_up);		break;
		case 6: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_down, sh_y_up, sh_z_up);		break;
		case 7: newGradientValues = n_DataSet->VoxelDataset->getVoxelGradient(sh_x_up, sh_y_up, sh_z_up);		break;
		}	

		if (grMode == GradientEstimationSW::BY_DIFFERENCES)
			temp_Gradient = Gradient::CreateNormalizedGradient(new Gradient(newGradientValues, calcGradient->scaleFactorDifferences));
		 else if (grMode == GradientEstimationSW::BY_4DREGRESSION)
			temp_Gradient = Gradient::CreateNormalizedGradient(new Gradient(newGradientValues, calcGradient->scaleFactorRegression));
		 temp_Gradient->scale_Length();
	}

	return temp_Gradient;
}
