#include "stdafx.h"
#include "Histogram.h"

using namespace System;
using namespace System::Collections;	
using namespace IDVR::Volume::DataStructures;
 	
/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
Histogram::Histogram(void) 
{
	this->m_DensityFrequency = NULL;
	this->m_MaxValue = this->m_Length = 0;
}

/*
Constructor initialization with the maximum number of valid gray (<i>density</i>) values.
All member variables will be set to zero, the Histogram::m_DensityFrequency member will be initialized with
the given maximum number of gray values.
*/
Histogram::Histogram(int numOfGrayvalues) 
{ 
	this->m_Length = numOfGrayvalues; 
	this->m_MaxValue = 0;
    this->m_DensityFrequency = new int __gc[numOfGrayvalues];
}

/*
Constructor initialization by an user-defined histogram.
The new instance will be set to the corresponding user-defined histogram values.
*/
Histogram::Histogram(int n_DensityFrequency __gc[], int n_Length) 
{ 
	this->m_DensityFrequency = n_DensityFrequency; 
	this->m_Length = n_Length;
	this->m_MaxValue = 0;
	for (int i = 0; i < this->m_Length; i++)
		this->m_MaxValue = Math::Max(this->m_MaxValue, m_DensityFrequency[i]);		
}

/*
This is the standard deconstructor.
Espacially the Histogram::m_DensityFrequency member which actual is an array will be deleted.
*/
Histogram::~Histogram(void)
{
	this->m_DensityFrequency = NULL;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
Increments the count of the specific density value.
The current histogram will not be changed besides the particularly choosen density value.
*/
void Histogram::addNewValue(unsigned short n_Density) 
{ 
	this->m_DensityFrequency[n_Density]++; 
	this->m_MaxValue = Math::Max(this->m_MaxValue, this->m_DensityFrequency[n_Density]);
} 

/*
Sets the choosen value to the specific density value.
The current histogram will not be changed besides the particularly choosen density value.
*/
void Histogram::addNewValue(unsigned short n_Density, int n_Value) 
{ 
	this->m_DensityFrequency[n_Density] = n_Value; 
	this->m_MaxValue = Math::Max(this->m_MaxValue, n_Value);
} 

//Returns the current density count of the specified density value.
int Histogram::getHistogramValue(unsigned short n_Density)
{ 
	return this->m_DensityFrequency[n_Density]; 
}

//Returns the current density histogram.
int Histogram::get_HistogramValues(void) __gc[]
{ 
	return this->m_DensityFrequency; 
}

//Returns the current density histogram length.
int Histogram::get_HistogramLength(void) 
{ 
	return this->m_Length; 
}

//Returns the maximum voxel count of the current histogram.
int Histogram::get_MaxVoxelCount(void) 
{
	return this->m_MaxValue; 
}

//Returns a new downsampled histogam by the chossen sample factor.
Histogram* Histogram::getDownSampledHistogramValues(int n_SampleFactor)
{
	int temp_value = 0;
	Histogram *downSampledValues = new Histogram(this->m_Length / n_SampleFactor);

	for (int i = 8; i < m_Length; i++)
	{
		if ((i != 8) && (i % n_SampleFactor == 0))
		{	
			downSampledValues->addNewValue((i / n_SampleFactor) - 2,
				Convert::ToInt32(Math::Round((double)temp_value / n_SampleFactor)));
			temp_value = 0;
		}
		temp_value += m_DensityFrequency[i];
	}
	downSampledValues->addNewValue((i / n_SampleFactor) - 1,
			Convert::ToInt32(Math::Round((double)temp_value / n_SampleFactor)));
	return downSampledValues;
}

