#pragma once

#include ".\RaySample.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace System::Drawing;

/*
Interface for the compositing model.

namespace: IDVR::Interfaces

This interface provides methods to blend the color and opacity values along a ray to obtain the final pixel color.
This is the third rendering step which consists of three steps: classification - shading - compositing
*/

public __gc __interface ICompositingModel
{
	/*
	Blends the final intensity and opacity values of a given raysample.
	*/
	void doCompositing(RaySample* n_RaySample) = 0;
	/*
	Blends the final pixelcolor with the given background color.
	*/
	void doCompositingBackground(float n_BackColor __gc[]) = 0;
	/*
	Resets the values of the composition model.
	*/
	void resetValues(void) = 0;

	/*
	Sets the intensity array.
	*/
	__property void set_Intensity(float n_Intensity __gc[]) = 0;
	/*
	Sets the opacity value.
	*/
	__property void set_Opacity(float n_Opacity) = 0;

	/*
	Returns the intensity array.
	*/
	__property float get_Intensity(void) __gc[] = 0;
	/*
	Returns the opacity value.
	*/
	__property float get_Opacity(void) = 0;
};
}
}

/*! \interface IDVR::Interfaces::ICompositingModel
	\brief <B>Interface for the compositing model.</B>

	namespace: IDVR::Interfaces

	This interface provides methods to blend the color and opacity values along a ray to obtain the final pixel color.
	This is the third rendering step which consists of three steps: classification - shading - compositing

	@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::DvrCompositingModelSW - the dvr compositing model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::LMipCompositingModelSW - the LMip compositing model

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-18-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>void doCompositing(RaySample* n_RaySample)</b><br><br>
	 Blends the final intensity and opacity values of a given raysample.<br><br>
	 \param n_RaySample the raysample that should be blended.</li><br><br>
	<li><b>void doCompositingBackground(float n_BackColor __gc[])</b><br><br>
	 Blends the final pixelcolor with the given background color.<br><br>
	 \param n_BackColor the background color which should be blended.</li><br><br>
	<li><b>void resetValues(void)</b><br><br>
	 Resets the values of the composition model.</li><br><br>
	<li><b>__property void set_Intensity(float n_Intensity __gc[])</b><br><br>
	 Sets the intensity array.<br><br>
	 \param n_Intensity the intensity array.</li><br><br>
	<li><b>__property void set_Opacity(float n_Opacity)</b><br><br>
	 Sets the opacity value.<br><br>
	 \param n_Opacity the opacity value.</li><br><br>
	<li><b>__property float get_Intensity(void) __gc[]</b><br><br>
	 Returns the intensity array.<br><br>
	 \return The intensity array.</li><br><br>
	<li><b>__property float get_Opacity(void)</b><br><br>
	 Returns the opacity value.<br><br>
	 \return The opacity value.</li>
   </ul>
 */