#pragma once

#include ".\VolumeData.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace System;
using namespace IDVR::Tools;
using namespace IDVR::Volume::DataStructures;

/*
Interface for the gradient estimation model.

namespace: IDVR::Interfaces

This interface provides methods to estimate the gradient either by central differences, or by 4d linear regression.
*/

public __gc __interface IGradientEstimation
{
public:

	/*
	Calculates the gradient by central differendes.
	*/
	float calculate_NormalEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[] = 0;
	/*
	Calculates the gradient by 4d inear regression.
	*/
	float calculate_4DLinerRegressionEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[] = 0;
};
}
}

/*! \interface IDVR::Interfaces::IGradientEstimation
	\brief <B>Interface for the gradient estimation model.</B>

	namespace: IDVR::Interfaces

	This interface provides methods to estimate the gradient either by central differences, or by 4d linear regression.

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-19-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>float calculate_NormalEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[]</b><br><br>
	 Calculates the gradient by 4d inear regression.<br><br>
	 \param n_Column column of the gradient.<br><br>
	 \param n_Row row of the gradient.<br><br>
	 \param n_Depth depth of the gradient.<br><br>
	 \param n_DataSet the dataset.<br><br>
	 \return the estimated gradient.</li><br><br>
	<li><b>float calculate_4DLinerRegressionEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[]</b><br><br>
	 Blends the final pixelcolor with the given background color.<br><br>
	 \param n_Column column of the gradient.<br><br>
	 \param n_Row row of the gradient.<br><br>
	 \param n_Depth depth of the gradient.<br><br>
	 \param n_DataSet the dataset.<br><br>
	 \return the estimated gradient.</li><br><br>
   </ul>
 */