#pragma once

#include "RaySample.h"
#include "VolumeData.h"
#include "Gradient.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::Tools;

/*
Interface for the gradient interpolation model.

namespace: IDVR::Interfaces

This interface provides methods to interpolate the gradient of a given raysample.
*/

public __gc __interface IGradientInterpolation
{
	/*
	Interpolates the gradient of a given raysample.
	\param intMode the interpolation mode.
	\param grMode the gradient estimation mode.
	\param n_RaySample the given raysample.
	\param n_DataSet the dataset.
	\param x_down x-value round down.
	\param x_up x-value round up.
	\param y_down y-value round down.
	\param y_up y-value round up.
	\param z_down z-value round down.
	\param z_up z-value round up.
	@return The interpolated gradient.
	*/
	Gradient* interpolateGradient(unsigned char intMode, unsigned char grMode, RaySample* n_RaySample, VolumeData* n_DataSet,
									double x_down, double x_up, double y_down, double y_up, double z_down, double z_up) = 0;
};
}
}

/*! \interface IDVR::Interfaces::IGradientInterpolation
	\brief <B>Interface for the gradient interpolation model.</B>

	namespace: IDVR::Interfaces

	This interface provides methods to interpolate the gradient of a given raysample.

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-19-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>Gradient* interpolateGradient(unsigned char intMode, unsigned char grMode, RaySample* n_RaySample, VolumeData* n_DataSet,
									double x_down, double x_up, double y_down, double y_up, double z_down, double z_up)</b><br><br>
	 Interpolates the gradient of a given raysample.<br><br>
	 \param intMode the interpolation mode.<br><br>
	 \param grMode the gradient estimation mode.<br><br>
	 \param n_RaySample the given raysample.<br><br>
	 \param n_DataSet the dataset.<br><br>
	 \param x_down x-value round down.<br><br>
	 \param x_up x-value round up.<br><br>
	 \param y_down y-value round down.<br><br>
	 \param y_up y-value round up.<br><br>
	 \param z_down z-value round down.<br><br>
	 \param z_up z-value round up.<br><br>
	 \return The interpolated gradient.</li><br><br>
   </ul>
 */