#pragma once

#include ".\RaySample.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace System::Collections;

/*
Interface for the IDVRCompositingModel.

namespace: IDVR::Interfaces

This interface provides methods needed for importance driven volume rendering. In the (see IIDVRCompositingModel::doCompositing) method,
it assigns a level of sparseness list for the objectIDs of a given ray. The (see IIDVRCompositingModel::modifyRaySample) method modifies
a given raysample according to a IDVRModulationModel.
*/

public __gc __interface IIDVRCompositingModel
{
	/*
	Assigns a level of sparseness list for the objectIDs of a given ray.
	*/
	void doCompositing(void) = 0;
	/*
	Modifies a given raysample.
	*/
	RaySample* modifyRaySample(RaySample *n_RaySample) = 0;
	/*
	Sets the list of objectIDs that occur on the ray.
	*/
	void setFootprintArrayList(ArrayList *n_Footprint) = 0;
};
}
}

/*! \interface IDVR::Interfaces::IIDVRCompositingModel
	\brief <B>Interface for the IDVRCompositingModel.</B>

	namespace: IDVR::Interfaces

	This interface provides methods needed for importance driven volume rendering. In the (see IIDVRCompositingModel::doCompositing) method,
	it assigns a level of sparseness list for the objectIDs of a given ray. The (see IIDVRCompositingModel::modifyRaySample) method modifies
	a given raysample according to a IDVRModulationModel.

	@see IDVR::VolumeRenderer::SWVolumeRenderer::IDVRcompositingModels::IDVRMImPCompositingModelSW - the maximum importance driven volume rendering model
	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-18-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>void doCompositing(void)</b><br><br>
	 Assigns a level of sparseness list for the objectIDs of a given ray.</li><br><br>
	<li><b>RaySample* modifyRaySample(RaySample *n_RaySample)</b><br><br>
	 Modifies a given raysample.<br><br>
	 \param n_RaySample the raysample that should be modified.<br><br>
	 \return The modified raysample.</li><br><br>
	<li><b>void setFootprintArrayList(ArrayList *n_Footprint)</b><br><br>
	 Sets the list of objectIDs that occur on the ray.<br><br>
	 \param n_Footprint arraylist of objectIDs.</li><br><br>
   </ul>
 */