#include "StdAfx.h"
#include ".\imageplane.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Tools;

ImagePlane::ImagePlane(void)
{
	this->m_ViewDirection = NULL;
	this->m_DefaultPoints = NULL;
	this->m_TransformedPoints = NULL;
	this->m_NullPointMiddlePoint = NULL;
	this->m_NullPoint = NULL;
	this->m_Resolution = 0;
}

ImagePlane::ImagePlane(Vector3* n_LeftUpper, Vector3* n_RightUpper, Vector3* n_LeftBottom, Vector3* n_RightBottom, short n_Resolution, unsigned short n_VolumeDimension __gc[])
{
	this->m_DefaultPoints = new Vector3 __gc*[4];
	this->m_TransformedPoints = new Vector3 __gc*[4];

	this->m_DefaultPoints[LUpper]  = this->m_TransformedPoints[LUpper] = n_LeftUpper;
	this->m_DefaultPoints[RUpper]  = this->m_TransformedPoints[RUpper] = n_RightUpper;
	this->m_DefaultPoints[LBottom]  = this->m_TransformedPoints[LBottom] = n_LeftBottom;
	this->m_DefaultPoints[RBottom]  = this->m_TransformedPoints[RBottom] = n_RightBottom;
	this->m_Resolution = n_Resolution;
	
	this->m_Width = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_DefaultPoints[RUpper], this->m_DefaultPoints[LUpper])->magnitude()));
	this->m_Heigth = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_DefaultPoints[LBottom], this->m_DefaultPoints[LUpper])->magnitude()));

	this->m_VolumeDimension = n_VolumeDimension;

	calculateRayDistance();

	calculateViewDirection();

	calculateNullPoint();

	calculateNullPointMiddlePoint();

	this->m_TranslateMatrix = Matrix4::CreateIdentityMatrix();
	this->m_RotMatrix = Matrix4::CreateIdentityMatrix();
	this->m_ScaleMatrix = Matrix4::CreateIdentityMatrix();
}

ImagePlane::ImagePlane(Vector3* n_LeftUpper, Vector3* n_RightUpper, Vector3* n_LeftBottom, Vector3* n_RightBottom, short n_Resolution, Vector3* n_ViewDirection, unsigned short n_VolumeDimension __gc[])
{
	this->m_DefaultPoints = new Vector3 __gc*[4];
	this->m_TransformedPoints = new Vector3 __gc*[4];

	this->m_DefaultPoints[LUpper]  = this->m_TransformedPoints[LUpper] = n_LeftUpper;
	this->m_DefaultPoints[RUpper]  = this->m_TransformedPoints[RUpper] = n_RightUpper;
	this->m_DefaultPoints[LBottom]  = this->m_TransformedPoints[LBottom] = n_LeftBottom;
	this->m_DefaultPoints[RBottom]  = this->m_TransformedPoints[RBottom] = n_RightBottom;
	this->m_Resolution = n_Resolution;

	this->m_ViewDirection = Vector3::CreateNormalizedVector(n_ViewDirection);

	this->m_Width = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_DefaultPoints[RUpper], this->m_DefaultPoints[LUpper])->magnitude()));
	this->m_Heigth = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_DefaultPoints[LBottom], this->m_DefaultPoints[LUpper])->magnitude()));

	this->m_VolumeDimension = n_VolumeDimension;

	calculateNullPoint();

	calculateRayDistance();

	calculateNullPoint();

	calculateNullPointMiddlePoint();

	this->m_TranslateMatrix = Matrix4::CreateIdentityMatrix();
	this->m_RotMatrix = Matrix4::CreateIdentityMatrix();
	this->m_ScaleMatrix = Matrix4::CreateIdentityMatrix();
}

ImagePlane::ImagePlane(ImagePlane* n_ImagePlane)
{
	this->m_Heigth = n_ImagePlane->Heigth;
	this->m_Width = n_ImagePlane->Width;
	this->m_Resolution = n_ImagePlane->Resolution;
	this->m_RayDistanceHeigth = n_ImagePlane->RayDistanceHeigth;
	this->m_RayDistanceWidth = n_ImagePlane->RayDistanceWidth;
	this->m_VolumeDimension =n_ImagePlane->VolumeDimension;

	this->m_DefaultPoints = n_ImagePlane->DefaultPoints;
	this->m_TransformedPoints = n_ImagePlane->TransformedPoints;
	this->m_ViewDirection = n_ImagePlane->ViewDirection;

	this->m_NullPoint = n_ImagePlane->NullPoint;
	this->m_NullPointMiddlePoint = n_ImagePlane->NullPointMiddlePoint;
	
	set_TranslateMatrix(n_ImagePlane->TranslateMatrix);
	set_RotMatrix(n_ImagePlane->RotMatrix);
	set_ScaleMatrix(n_ImagePlane->ScaleMatrix);
}

double ImagePlane::get_Width(void)
{
	return this->m_Width;
}

double ImagePlane::get_Heigth(void)
{
	return this->m_Heigth;
}

double ImagePlane::get_Resolution(void)
{
	return this->m_Resolution;
}

unsigned short ImagePlane::get_VolumeDimension(void) __gc[]
{
	return this->m_VolumeDimension;
}

Vector3* ImagePlane::get_ViewDirection(void)
{
	return this->m_ViewDirection;
}

double ImagePlane::get_RayDistanceWidth(void)
{
	return this->m_RayDistanceWidth;
}

double ImagePlane::get_RayDistanceHeigth(void)
{
	return this->m_RayDistanceHeigth;
}

Vector3* ImagePlane::get_DefaultPoints(void) []
{
	return this->m_DefaultPoints;
}

Vector3* ImagePlane::get_TransformedPoints(void) []
{
	return this->m_TransformedPoints;
}

Matrix4* ImagePlane::get_TranslateMatrix(void)
{
	return this->m_TranslateMatrix;
}

Matrix4* ImagePlane::get_RotMatrix(void)
{
	return this->m_RotMatrix;
}

Matrix4* ImagePlane::get_ScaleMatrix(void)
{
	return this->m_ScaleMatrix;
}

void ImagePlane::set_Resolution(double n_Resolution)
{
	this->m_Resolution = n_Resolution;
}

void ImagePlane::set_ViewDirection(Vector3* n_ViewDirection)
{
	this->m_ViewDirection = n_ViewDirection;
}

Vector3* ImagePlane::get_NullPoint(void)
{
	return this->m_NullPoint;
}

Vector3* ImagePlane::get_NullPointMiddlePoint(void)
{
	return this->m_NullPointMiddlePoint;
}

Vector3* ImagePlane::getCurrentLeftUpperPoint(void)
{
	return this->m_TransformedPoints[LUpper];
}

Vector3* ImagePlane::getCurrentRightBottomPoint(void)
{
	return this->m_TransformedPoints[RBottom];
}

Vector3* ImagePlane::getCurrentLeftBottomPoint(void)
{
	return this->m_TransformedPoints[LBottom];
}

Vector3* ImagePlane::getCurrentRightUpperPoint(void)
{
	return this->m_TransformedPoints[RUpper];
}

Vector3* ImagePlane::getDefaultLeftUpperPoint(void)
{
	return this->m_DefaultPoints[LUpper];
}

Vector3* ImagePlane::getDefaultRightBottomPoint(void)
{
	return this->m_DefaultPoints[RBottom];
}

Vector3* ImagePlane::getDefaultLeftBottomPoint(void)
{
	return this->m_DefaultPoints[LBottom];
}

Vector3* ImagePlane::getDefaultRightUpperPoint(void)
{
	return this->m_DefaultPoints[RUpper];
}

Vector3* ImagePlane::getDefaultPoints(void) __gc[]
{
	return this->m_DefaultPoints;
}

Vector3* ImagePlane::getHorizontalViewVector(void)
{
	Vector3 *temp = Vector3::CreateNormalizedVector(Vector3::Subtraction(this->m_TransformedPoints[RUpper], this->m_TransformedPoints[LUpper]));
	temp = temp->multSkalar(this->m_RayDistanceWidth);
	return temp;
}

Vector3* ImagePlane::getVerticalViewVector(void)
{
	Vector3 *temp = Vector3::CreateNormalizedVector(Vector3::Subtraction(this->m_TransformedPoints[LUpper], this->m_TransformedPoints[LBottom]));
	return temp->multSkalar(this->m_RayDistanceHeigth);
}

Vector3* ImagePlane::getDistanceFromOrigin(void)
{
	return this->m_NullPointMiddlePoint;
}

void ImagePlane::set_Width(double n_Width)
{
	this->m_Width = n_Width;
}

void ImagePlane::set_Heigth(double n_Heigth)
{
	this->m_Heigth = n_Heigth;
}

void ImagePlane::set_VolumeDimension(unsigned short n_VolumeDimension __gc[])
{
	this->m_VolumeDimension = n_VolumeDimension;
}

void ImagePlane::set_TranslateMatrix(Matrix4 *n_TranslateMatrix)
{
	this->m_TranslateMatrix = new Matrix4(n_TranslateMatrix);
}

void ImagePlane::set_RotMatrix(Matrix4 *n_RotMatrix)
{
	this->m_RotMatrix = new Matrix4(n_RotMatrix);
}

void ImagePlane::set_ScaleMatrix(Matrix4 *n_ScaleMatrix)
{
	this->m_ScaleMatrix = new Matrix4(n_ScaleMatrix);
}

void ImagePlane::setDefaultLeftUpperPoint(Vector3* n_LeftUpper)
{
	this->m_DefaultPoints[LUpper] = n_LeftUpper;
}

void ImagePlane::setDefaultRightBottomPoint(Vector3* n_RightBottom)
{
	this->m_DefaultPoints[RBottom] = n_RightBottom;
}

void ImagePlane::setDefaultLeftBottomPoint(Vector3* n_LeftBottom)
{
	this->m_DefaultPoints[LBottom] = n_LeftBottom;
}

void ImagePlane::setDefaultRightUpperPoint(Vector3* n_RightUpper)
{
	this->m_DefaultPoints[RBottom] = n_RightUpper;
}

void ImagePlane::calculateRayDistance(void)
{
	this->m_RayDistanceWidth = (double)this->m_Width / this->m_Resolution;
	this->m_RayDistanceHeigth = (double)this->m_Heigth / this->m_Resolution;
}

void ImagePlane::calculateViewDirection(void)
{
	this->m_ViewDirection = Vector3::CreateNormalizedVector(Vector3::CrossProd(this->getHorizontalViewVector(), this->getVerticalViewVector()));
}

void ImagePlane::transformImagePlane(void)
{
	resetPoints();

	Matrix4 *transformMatrix = Matrix4::CreateTranslationMatrix(this->m_NullPointMiddlePoint);
	
	transformMatrix = Matrix4::MultiplyMatices(transformMatrix, this->m_TranslateMatrix);
	transformMatrix = Matrix4::MultiplyMatices(transformMatrix, this->m_RotMatrix);
	transformMatrix = Matrix4::MultiplyMatices(transformMatrix, this->m_ScaleMatrix);
	
	transformMatrix = Matrix4::MultiplyMatices(transformMatrix, Matrix4::CreateTranslationMatrix(this->m_NullPointMiddlePoint->getNegativeVector()));
	
	this->m_TransformedPoints[LUpper]  =  Vector4::GetVector3(transformMatrix->multiply_WithPoint(new Vector4(this->m_DefaultPoints[LUpper], 1.0)));
	this->m_TransformedPoints[LBottom] = Vector4::GetVector3(transformMatrix->multiply_WithPoint(new Vector4(this->m_DefaultPoints[LBottom], 1.0)));
	this->m_TransformedPoints[RUpper]  =  Vector4::GetVector3(transformMatrix->multiply_WithPoint(new Vector4(this->m_DefaultPoints[RUpper], 1.0)));
	this->m_TransformedPoints[RBottom] = Vector4::GetVector3(transformMatrix->multiply_WithPoint(new Vector4(this->m_DefaultPoints[RBottom], 1.0)));
	
	calculateNewParameters();
}

void ImagePlane::calculateNewParameters(void)
{
	calculateViewDirection();

	this->m_Width = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_TransformedPoints[RUpper], this->m_TransformedPoints[LUpper])->magnitude()));
	this->m_Heigth = Convert::ToInt16(Math::Round(Vector3::Subtraction(this->m_TransformedPoints[LUpper], this->m_TransformedPoints[LBottom])->magnitude()));

	calculateRayDistance();
}

void ImagePlane::calculateNullPointMiddlePoint(void)
{
	calculateNullPoint();
	this->m_NullPointMiddlePoint = Vector3::Subtraction(calculateDefaultMiddlePoint(), this->m_NullPoint);
}

Vector3* ImagePlane::calculateMiddlePoint(void)
{
	Vector3 *dirHor = Vector3::Subtraction(this->m_TransformedPoints[RUpper], this->m_TransformedPoints[LUpper]);
	Vector3 *dirVer = Vector3::Subtraction(this->m_TransformedPoints[LUpper], this->m_TransformedPoints[LBottom]);
	return Vector3::Addition(Vector3::Addition(this->m_TransformedPoints[LBottom], dirHor)->multSkalar(0.5),
							(Vector3::Addition(this->m_TransformedPoints[LBottom], dirVer)->multSkalar(0.5)));
}

Vector3* ImagePlane::calculateDefaultMiddlePoint(void)
{
	Vector3 *dirHor = Vector3::Subtraction(this->m_DefaultPoints[RUpper], this->m_DefaultPoints[LUpper]);
	Vector3 *dirVer = Vector3::Subtraction(this->m_DefaultPoints[LUpper], this->m_DefaultPoints[LBottom]);
	return Vector3::Addition(Vector3::Addition(this->m_DefaultPoints[LBottom], dirHor)->multSkalar(0.5),
							(Vector3::Addition(this->m_DefaultPoints[LBottom], dirVer)->multSkalar(0.5)));
}

double ImagePlane::calculateDistance(Vector3 *n_Point)
{
	double D = - (this->m_ViewDirection->X * this->m_TransformedPoints[0]->X) - (this->m_ViewDirection->Y * m_TransformedPoints[0]->Y)
			   - (this->m_ViewDirection->Z * m_TransformedPoints[0]->Z);

	return (this->m_ViewDirection->X * n_Point->X) + (this->m_ViewDirection->Y * n_Point->Y) 
			+ (this->m_ViewDirection->Z * n_Point->Z) + D;
}

void ImagePlane::calculateNullPoint(void)
{
	Vector3 *startPoint = new Vector3(0.0, 0.0, 0.0);

	Vector3 *volumeMiddlePoint = new Vector3(this->m_VolumeDimension[0] / 2, this->m_VolumeDimension[1] / 2,this->m_VolumeDimension[2] / 2);
	double distance = calculateDistance(volumeMiddlePoint);

	this->m_NullPoint = Vector3::Addition(startPoint, Vector3::CreateNormalizedVector(this->m_ViewDirection->getNegativeVector())->multSkalar(distance));

	int stop = 0;
}

void ImagePlane::resetPoints()
{
	this->m_TransformedPoints[LUpper] = this->m_DefaultPoints[LUpper];
	this->m_TransformedPoints[RUpper] = this->m_DefaultPoints[RUpper];
	this->m_TransformedPoints[LBottom] = this->m_DefaultPoints[LBottom];
	this->m_TransformedPoints[RBottom] = this->m_DefaultPoints[RBottom];
}

void ImagePlane::resetMatrices(void)
{
	this->m_TranslateMatrix = Matrix4::CreateIdentityMatrix();
	this->m_RotMatrix = Matrix4::CreateIdentityMatrix();
	this->m_ScaleMatrix = Matrix4::CreateIdentityMatrix();
}

void ImagePlane::modifyTranslateMatrix(Matrix4 *n_TranslateMatrix)
{
	this->m_TranslateMatrix = Matrix4::MultiplyMatices(this->m_TranslateMatrix, n_TranslateMatrix);
}

void ImagePlane::modifyRotMatrix(Matrix4 *n_RotMatrix)
{
	this->m_RotMatrix = Matrix4::MultiplyMatices(n_RotMatrix, this->m_RotMatrix);
	//this->m_RotMatrix = Matrix4::MultiplyMatices(this->m_RotMatrix, n_RotMatrix);
}

void ImagePlane::modifyScaleMatrix(Matrix4 *n_ScaleMatrix)
{
	this->m_ScaleMatrix = Matrix4::MultiplyMatices(n_ScaleMatrix, this->m_ScaleMatrix);
}

bool ImagePlane::sameSide(Vector3* n_Vector1, Vector3 *n_Vector2)
{
	Vector3 *v1 = getHorizontalViewVector();
	Vector3 *v2 = getVerticalViewVector();

	Vector3* cross = Vector3::CrossProd(v1, v2);

	double A, B, C, D;

	A = cross->X;
	B = cross->Y;
	C = cross->Z;

	D = Vector3::DotProd(cross, this->m_TransformedPoints[LBottom]);

	//hesse form

	double factor = (new Vector3(A, B, C))->magnitude();

	if(D > 0)
	{
		A = -A;
		B = -B;
		C = -C;
		D = -D;
	}

	A = A / factor;
	B = B / factor;
	C = C / factor;
	D = D / factor;

	double distance1 = A * n_Vector1->X + B * n_Vector1->Y + C * n_Vector1->Z + D;
	double distance2 = A * n_Vector2->X + B * n_Vector2->Y + C * n_Vector2->Z + D;

	return Math::Sign(distance1) == Math::Sign(distance2);
}