#include "StdAfx.h"
#include ".\InterpolationSW.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;

//Standard constructor.
InterpolationSW::InterpolationSW(void)
{
}

//Interpolates the density.
unsigned short InterpolationSW::interpolate_Density(unsigned char intMode, RaySample* n_RaySample, VolumeData* n_DataSet, 
													double x_down, double x_up, double y_down, double y_up, double z_down, double z_up)
{
	unsigned short temp_Density;

	unsigned short	sh_x_down = (unsigned short)x_down,
					sh_x_up =   (unsigned short)x_up,
					sh_y_down = (unsigned short)y_down,
					sh_y_up =   (unsigned short)y_up,
					sh_z_down = (unsigned short)z_down,
					sh_z_up =   (unsigned short)z_up;

	if(intMode == InterpolationSW::BY_TRILINIEAR)
	{
		double x = n_RaySample->get_Coords()->X;
		double y = n_RaySample->get_Coords()->Y;
		double z = n_RaySample->get_Coords()->Z;

		//Trilinear interpolation of the neighbourhood densities
		temp_Density = (Convert::ToInt16(Math::Round(
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down, sh_y_down, sh_z_down)  * (x_up - x)   * (y_up - y)   * (z_up - z))   +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,   sh_y_down, sh_z_down)  * (x - x_down) * (y_up - y)   * (z_up - z))   +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down, sh_y_up,   sh_z_down)  * (x_up - x)   * (y - y_down) * (z_up - z))   +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,   sh_y_up,   sh_z_down)  * (x - x_down) * (y - y_down) * (z_up - z))   +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down, sh_y_down, sh_z_up)    * (x_up - x)   * (y_up - y)   * (z - z_down)) +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,   sh_y_down, sh_z_up)    * (x - x_down) * (y_up - y)   * (z - z_down)) +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down, sh_y_up,   sh_z_up)    * (x_up - x)   * (y - y_down) * (z - z_down)) +
					(double)(n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,   sh_y_up,   sh_z_up)    * (x - x_down) * (y - y_down) * (z - z_down)))));
	}
	
	else if(intMode == InterpolationSW::BY_NEARESTNEIGHBOUR)
	{
		//Selection of sample`s objectID by nearest neighbour choice
		double temp_d __gc[] = new double __gc[8], distance = Double::MaxValue; int selectedIndex = 0;
		temp_d[0] = Vector3::Subtraction(new Vector3(x_down, y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[1] = Vector3::Subtraction(new Vector3(x_up, y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[2] = Vector3::Subtraction(new Vector3(x_down, y_up, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[3] = Vector3::Subtraction(new Vector3(x_up, y_up, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[4] = Vector3::Subtraction(new Vector3(x_down, y_down, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[5] = Vector3::Subtraction(new Vector3(x_up, y_down, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[6] = Vector3::Subtraction(new Vector3(x_down, y_up, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[7] = Vector3::Subtraction(new Vector3(x_up, y_up, z_up), n_RaySample->get_Coords())->magnitude();

		//get the index width the smallest distance
		for (int index = 0; index < 8; index++)
			if (temp_d[index] < distance) 
			{
				distance = temp_d[index]; 
				selectedIndex = index;
			}

		switch(selectedIndex) 
		{
		case 0: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down,	sh_y_down,	sh_z_down);		break;
		case 1: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,	sh_y_down,	sh_z_down);		break;
		case 2: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down,	sh_y_up,	sh_z_down);		break;
		case 3: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,	sh_y_up,	sh_z_down);		break;
		case 4: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down,	sh_y_down,	sh_z_up);	    break;
		case 5: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,	sh_y_down,	sh_z_up);	    break;
		case 6: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_down,	sh_y_up,	sh_z_up);	    break;
		case 7: temp_Density = n_DataSet->VoxelDataset->getVoxelDensity(sh_x_up,	sh_y_up,	sh_z_up);		break;
		default: break;		
		}
	}
	
	return temp_Density;
}

//Interpolates the objectID.
unsigned short InterpolationSW::interpolate_ObjectID(unsigned char intMode, RaySample* n_RaySample, VolumeData* n_DataSet,
											double x_down, double x_up, double y_down, double y_up, double z_down, double z_up)
{
	unsigned short temp_ObjectID;

	unsigned short	sh_x_down = (unsigned short)x_down,
					sh_x_up =   (unsigned short)x_up,
					sh_y_down = (unsigned short)y_down,
					sh_y_up =   (unsigned short)y_up,
					sh_z_down = (unsigned short)z_down,
					sh_z_up =   (unsigned short)z_up;

	if(intMode == InterpolationSW::BY_NEARESTNEIGHBOUR)
	{
		//Selection of sample`s objectID by nearest neighbour choice
		double temp_d __gc[] = new double __gc[8], distance = Double::MaxValue; int selectedIndex = 0;
		temp_d[0] = Vector3::Subtraction(new Vector3(x_down, y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[1] = Vector3::Subtraction(new Vector3(x_up, y_down, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[2] = Vector3::Subtraction(new Vector3(x_down, y_up, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[3] = Vector3::Subtraction(new Vector3(x_up, y_up, z_down), n_RaySample->get_Coords())->magnitude();
		temp_d[4] = Vector3::Subtraction(new Vector3(x_down, y_down, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[5] = Vector3::Subtraction(new Vector3(x_up, y_down, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[6] = Vector3::Subtraction(new Vector3(x_down, y_up, z_up), n_RaySample->get_Coords())->magnitude();
		temp_d[7] = Vector3::Subtraction(new Vector3(x_up, y_up, z_up), n_RaySample->get_Coords())->magnitude();

		//get the index width the smallest distance
		for (int index = 0; index < 8; index++)
			if (temp_d[index] < distance) 
			{
				distance = temp_d[index]; 
				selectedIndex = index;
			}

		switch(selectedIndex) 
		{
			case 0: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down,	sh_y_down,	sh_z_down);	break;
			case 1: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,		sh_y_down,	sh_z_down);	break;
			case 2: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down,	 sh_y_up,   sh_z_down);	break;
			case 3: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,		sh_y_up,	sh_z_down);	break;
			case 4: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down,	sh_y_down,	sh_z_up);	break;
			case 5: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,		sh_y_down,	sh_z_up);	break;
			case 6: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down,	sh_y_up,	sh_z_up);	break;
			case 7: temp_ObjectID = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,		sh_y_up,	sh_z_up);	break;
			default: break;		
		}
	}
	else if(intMode == InterpolationSW::BY_TRILINIEAR)
	{
		double x = n_RaySample->get_Coords()->X;
		double y = n_RaySample->get_Coords()->Y;
		double z = n_RaySample->get_Coords()->Z;

		//Selection of sample`s objectID by nearest neighbour choice
		unsigned short temp_IDs __gc[] = new unsigned short __gc[8], 
					   temp_ID = 0, 
					   selectedIndex = 0;
		double difference = Double::MaxValue;

		temp_IDs[0] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down, sh_y_down, sh_z_down);
		temp_IDs[1] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up  , sh_y_down, sh_z_down);
		temp_IDs[2] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down, sh_y_up,   sh_z_down);
		temp_IDs[3] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,   sh_y_up,   sh_z_down);
		temp_IDs[4] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down, sh_y_down, sh_z_up);
		temp_IDs[5] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,   sh_y_down, sh_z_up);
		temp_IDs[6] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_down, sh_y_up,   sh_z_up);
		temp_IDs[7] = n_DataSet->VoxelDataset->getVoxelObjectID(sh_x_up,   sh_y_up,   sh_z_up);

		//Trilinear interpolation of the neighbourhood ID's
		temp_ID = (Convert::ToInt16(Math::Round(
				(double)(temp_IDs[0]  *  (x_up - x)  *  (y_up - y)  *  (z_up - z))  +
				(double)(temp_IDs[1]  * (x - x_down) *  (y_up - y)  *  (z_up - z))  +
				(double)(temp_IDs[2]  *  (x_up - x)  * (y - y_down) *  (z_up - z))  +
				(double)(temp_IDs[3]  * (x - x_down) * (y - y_down) *  (z_up - z))  +
				(double)(temp_IDs[4]  *  (x_up - x)  *  (y_up - y)  * (z - z_down)) +
				(double)(temp_IDs[5]  * (x - x_down) *  (y_up - y)  * (z - z_down)) +
				(double)(temp_IDs[6]  *  (x_up - x)  * (y - y_down) * (z - z_down)) +
				(double)(temp_IDs[7]  * (x - x_down) * (y - y_down) * (z - z_down)))));

		//get the index width the smallest distance
		for (int index = 0; index < 8; index++)
		{
			double currentDifference = Math::Abs(temp_IDs[index] - temp_ID);
			if (currentDifference < difference) 
			{
				difference = currentDifference; 
				selectedIndex = index;
			}
		}

		temp_ObjectID = temp_IDs[selectedIndex];
	}

	return temp_ObjectID;
}
