#pragma once
#include "iinterpolation.h"
#include "GradientEstimationSW.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace RenderCalculations {

using namespace IDVR::Interfaces;

/**
<B>Class for the interpolation.</B>

namespace: IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations

Baseclass: IDVR::Interfaces::IInterpolation

This class provides methods to interpolate the density and objectID of a given raysample. Interpolation can be done
by trilinear interpolation or nearest neighbour.

@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-19-2005
*/

public __gc class InterpolationSW : public IInterpolation
{
public:

	/**
	Standard constructor.
	*/
	InterpolationSW(void);

	/**
	Interpolates the density.
	@param intMode the interpolation mode.
	@param n_RaySample the given raysample.
	@param n_DataSet the dataset.
	@param x_down x-value round down.
	@param x_up x-value round up.
	@param y_down y-value round down.
	@param y_up y-value round up.
	@param z_down z-value round down.
	@param z_up z-value round up.
	@return The interpolated value.
	*/
	virtual unsigned short interpolate_Density(unsigned char intMode, RaySample* n_RaySample, VolumeData* n_DataSet, 
											   double x_down, double x_up, double y_down, double y_up, double z_down, double z_up);
	
	/**
	Interpolates the objectID.
	@param intMode the interpolation mode.
	@param n_RaySample the given raysample.
	@param n_DataSet the dataset.
	@param x_down x-value round down.
	@param x_up x-value round up.
	@param y_down y-value round down.
	@param y_up y-value round up.
	@param z_down z-value round down.
	@param z_up z-value round up.
	@return The interpolated value.
	*/
	virtual unsigned short interpolate_ObjectID(unsigned char intMode, RaySample* n_RaySample, VolumeData* n_DataSet, 
											    double x_down, double x_up, double y_down, double y_up, double z_down, double z_up);

	static const unsigned char BY_NEARESTNEIGHBOUR = 0; /**< interpolation by nearest neighbour. */
	static const unsigned char BY_TRILINIEAR = 1; /**< trilinear interpolation. */
};
}
}
}
}
