#pragma once

#include "IShadingmodel.h"
#include "Vector3.h"
#include "Light.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ShadingModels {

using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Tools;

/**<B>Class for the mip shading model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels

Baseclass: IDVR::Interfaces::IShadingModel

This class calculates the color value of a given raysample according to their opacity value.

@see ContourShadingModelSW - the contour shading model
@see PhongShadingModelSW - the phong shading model
@see ToneShadingModelSW - the tone shading model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/
public __gc class MipShadingModelSW : public IShadingModel
{
private:

	__value enum colorIndices{RED, GREEN, BLUE};

	unsigned short m_DensityScope; /**< the scope of the density values of the dataset. */
	float m_BaseColor __gc[]; /**< the base color. */

public:

	/**
	Standard constructor.
	*/
	MipShadingModelSW(void);
	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	@param n_Col the base color.
	*/
	MipShadingModelSW(unsigned short n_DensityScope, Color n_Col);

	/**
	Calculates the Color value for a given raysample.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doShading(RaySample* n_RaySample);

	/**
	Returns the current color as Color object.
	@return The current color as Color object.
	*/
	__property Color get_Col(void);
};
}
}
}
}
