#include "StdAfx.h"
#include ".\PhongShadingModelSW.h"
#using <mscorlib.dll>

#include "DataLoader.h"

using namespace IDVR::Volume;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

//Standard constructor.
PhongShadingModelSW::PhongShadingModelSW(void)
{
	this->k_a = DataLoader::Default_Ambient; this->k_d = DataLoader::Default_Diffuse; 
	this->k_s = DataLoader::Default_Specular; this->n = DataLoader::Default_SpExponent;
	this->k_1 = DataLoader::Default_K1DepthCueing; this->k_2 = DataLoader::Default_K2DepthCueing; this->d = 0;
	this->L = this->V = this->H = NULL; 
	c_p = new float __gc[3];
	this->m_TransferFunc = NULL;
}

//Constructor.
PhongShadingModelSW::PhongShadingModelSW(double n_Ambient, double n_Diffuse, double n_Specular, double n_SpExponent)
{
	this->k_a = n_Ambient; this->k_d = n_Diffuse; this->k_s = n_Specular; this->n = n_SpExponent;
	this->k_1 = this->k_2 = this->d = 0;
	this->L = this->V = this->H = NULL; 
	c_p = new float __gc[3];
	this->m_TransferFunc = NULL;
}

//Constructor.
PhongShadingModelSW::PhongShadingModelSW(double n_Ambient, double n_Diffuse, double n_Specular, double n_SpExponent,
									 double n_DepthCue1, double n_DepthCue2, /*Vector3* n_ViewDirection, 
									 Vector3* n_ImagePlanePoint, Light* n_Light,*/ Transferfunction *n_TransferFunc)
{
	this->k_a = n_Ambient; this->k_d = n_Diffuse; this->k_s = n_Specular; this->n = n_SpExponent;
	this->k_1 = n_DepthCue1; this->k_2 = n_DepthCue2;

	this->L = this->V = this->H = NULL; 
	c_p = new float __gc[3];

	//c_p = new float __gc[3];
	//Color light = Color::FromArgb(n_Light->get_LightColor());
	//this->c_p[RED] = (float)light.get_R() / 255.0f; 
	//this->c_p[GREEN] = (float)light.get_G() / 255.0f; 
	//this->c_p[BLUE] = (float)light.get_B() / 255.0f;

	////reflect the y component of the current phong shading light direction
	////because the world coord system is composite to the coord system in which the
	////light direction has been calculated (n_light->LightDirection) about the y component.
	//// (see therefore the DataLoader class and the funtion readDataFile, especially the reverse iteration
	////  for reading the voxels into the data structure)
	//this->L = n_Light->getReflectedLightDirection(Matrix4::CreateReflectionAboutXMatrix());

	//this->V = n_ViewDirection; 

	//this->m_imgePlanePoint = n_ImagePlanePoint;
	this->m_TransferFunc = n_TransferFunc;

	/*calculateHighLightDirection();*/
}

//Calculates the Color value for a given raysample.
RaySample* PhongShadingModelSW::doShading(RaySample* n_RaySample)
{
	calculateVoxelToPlaneDistance(n_RaySample->get_Coords());

	float sampleColor __gc[] = this->m_TransferFunc->getColor(n_RaySample->get_Density());
	float shadedIntensity __gc[] = new float __gc[3];

	float lightReflection = (float)n_RaySample->get_Gradient()->get_GradientVector()->dotProd(this->L);

	shadedIntensity[RED] = 
	//ambient part
	(float)(this->k_a * this->c_p[RED]) * sampleColor[RED];

	shadedIntensity[GREEN] =
		//ambient part
		(float)(this->k_a * this->c_p[GREEN]) * sampleColor[GREEN];
		
	shadedIntensity[BLUE] = 
		//ambient part
		(float)(this->k_a * this->c_p[BLUE]) * sampleColor[BLUE];

	if (lightReflection > 0.0f) 
	{
		float fractionOfVisibleLight = 
			//diffuse part
			(float)(this->k_d * lightReflection) + 
			//specular part
			(float)(this->k_s *  Math::Pow(n_RaySample->get_Gradient()->get_GradientVector()->dotProd(this->H), this->n));

		shadedIntensity[RED] += 
			//depth cueing * fractionOfVisibleLight
			(float)((this->c_p[RED] / (this->k_1 + (this->k_2 * this->d))) * fractionOfVisibleLight);

		shadedIntensity[GREEN] +=
			//depth cueing * fractionOfVisibleLight
			(float)((this->c_p[GREEN] / (this->k_1 + (this->k_2 * this->d))) * fractionOfVisibleLight);

		shadedIntensity[BLUE] += 
			//depth cueing * fractionOfVisibleLight
			(float)((this->c_p[BLUE] / (this->k_1 + (this->k_2 * this->d))) * fractionOfVisibleLight);
	}

	if ( shadedIntensity[RED] > 1) shadedIntensity[RED] = 1;
	else if (shadedIntensity[RED] < 0) shadedIntensity[RED] = 0;
	if ( shadedIntensity[GREEN] > 1) shadedIntensity[GREEN] = 1;
	else if (shadedIntensity[GREEN] < 0) shadedIntensity[GREEN] = 0;
	if ( shadedIntensity[BLUE] > 1) shadedIntensity[BLUE] = 1;
	else if (shadedIntensity[BLUE] < 0) shadedIntensity[BLUE] = 0;

	n_RaySample->set_Color(shadedIntensity);
	return n_RaySample;
}

//Sets the current view direction.
void PhongShadingModelSW::set_ViewingDirection(Vector3* n_ViewDirection)
{
	this->V = n_ViewDirection;

	calculateHalfwayVector();
}

//Sets an arbitrary image plane point (needed for calculation of normal voxel to plane distance).
void PhongShadingModelSW::set_ImagePlanePoint(Vector3* n_ImagePlanePoint)
{
	this->m_imgePlanePoint = n_ImagePlanePoint;
}

//Sets the current light.
void PhongShadingModelSW::set_LightModel(Light* n_Light)
{
	Color light = Color::FromArgb(n_Light->get_LightColor());
	this->c_p[RED] = (float)light.get_R() / 255.0f; 
	this->c_p[GREEN] = (float)light.get_G() / 255.0f; 
	this->c_p[BLUE] = (float)light.get_B() / 255.0f;

	this->L = n_Light->OffsetLightDirection;

	calculateHalfwayVector();
}

//Returns the ambient value.
double PhongShadingModelSW::get_Ambient(void)
{
	return this->k_a;
}

//Returns the diffuse value.
double PhongShadingModelSW::get_Diffuse(void)
{
	return this->k_d;
}

//Returns the specular value.
double PhongShadingModelSW::get_Specular(void)
{
	return this->k_s;
}

//Returns the specular exponent.
double PhongShadingModelSW::get_SpExponent(void)
{
	return this->n;
}

//Returns the first value for depth cueing.
double PhongShadingModelSW::get_DepthK1(void)
{
	return this->k_1;
}

//Returns the second value for depth cueing.
double PhongShadingModelSW::get_DepthK2(void)
{
	return this->k_2;
}

//Returns the transfer function.
Transferfunction* PhongShadingModelSW::get_TransferFunc(void)
{
	return this->m_TransferFunc;
}

//Initialize viewing and lighting parameter all together to the given values.
void PhongShadingModelSW::setViewLightParameters(Vector3* n_ViewDirection, Vector3* n_ImagePlanePoint, Light* n_Light)
{
	Color light = Color::FromArgb(n_Light->get_LightColor());
	this->c_p[RED] = (float)light.get_R() / 255.0f; 
	this->c_p[GREEN] = (float)light.get_G() / 255.0f; 
	this->c_p[BLUE] = (float)light.get_B() / 255.0f;

	this->L = n_Light->OffsetLightDirection;
	this->V = n_ViewDirection;
	this->m_imgePlanePoint = n_ImagePlanePoint;

	calculateHalfwayVector();
}

//Calculates the normal voxel to plane distance of a given point.
void PhongShadingModelSW::calculateVoxelToPlaneDistance(Vector3* n_SampleCoords)
{
	double D = - (this->V->X * this->m_imgePlanePoint->X) - (this->V->Y * m_imgePlanePoint->Y)
			   - (this->V->Z * m_imgePlanePoint->Z);

	this->d = (this->V->X * n_SampleCoords->X) + (this->V->Y * n_SampleCoords->Y) 
			+ (this->V->Z * n_SampleCoords->Z) + D;
}

//Returns the halfway vector.
void PhongShadingModelSW::calculateHalfwayVector(void)
{
	this->H = Vector3::CreateNormalizedVector(Vector3::Addition(this->V, this->L));
}