#pragma once

#include ".\RaySample.h"

namespace IDVR {
namespace VolumeRenderer {
namespace RenderPrimitives {

	using namespace System::Collections;

/**
<b>Class for the ray data structure.</b>

This class contains a couple of methods for the ray object. Each ray object stores the resulting color
values for the different objects of one ray. This object is used for two level rendering, where the stored
color values are then blend together to obtain the final color of the ray.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-17-2005
*/
public __gc class Ray
{
private:
	ArrayList *m_RaySamples; /**< Stores the calculated raysamples. */
	ArrayList *m_UsedObjectIDs; /**< Stores the already used objectIDs of the raysamples. */

public:
	
	/**
	This is the standard constructor.
	*/
	Ray(void);
	/**
	This is the standard destructor.
	*/
	~Ray(void);

	/**
	Sets the raycapacity.
	@param n_RayCapacity the raycapacity.
	*/
	void setRayCapacity(int n_RayCapacity);
	/**
	Inserts a raysample into the ray.
	@param newSample the new raysample.
	*/
	void insertSample(RaySample* newSample);
	/**
	Returns the raysample at the given position.
	@param rayPosition the position of the raysample.
	@return The raysamle at the given position.
	*/
	RaySample* getRaySample(int rayPosition);
	/**
	Returns all raysamples of the ray.
	@return All raysamples of the ray.
	*/
	Array* getRaySamples(void);
	/**
	Returns the already used objectIDs of the raysamples that are stored in the ray.
	@return The already used objectIDs.
	*/
	unsigned short getUsedObjectIDs(void) __gc[];
	/**
	Returns the number of stored raysamples.
	@return The number of stored raysamples.
	*/
	int getSampleCount(void);
	/**
	Cleares all raysamples of the ray.
	*/
	void clearRay(void);
};
}
}
}

