#include "StdAfx.h"
#include ".\rendermachinesw.h"
#using <mscorlib.dll>

using namespace System::Threading;
using namespace System::Globalization;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer;
using namespace IDVR::Tools;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;
using namespace IDVR;

//Standard constructor.
RenderMachineSW::RenderMachineSW(void)
{
	this->m_ImagePlane = NULL;
	this->m_FrameBuffer = NULL;
	this->m_GlobalRenderer = NULL;
	this->m_DataSet = NULL;
	this->m_Canvas = NULL;
	this->m_GUIHandler = NULL;
	this->m_ProgressWin = NULL;
}

//Constructor.
RenderMachineSW::RenderMachineSW(VolumeData *n_Dataset, RenderPrimitives::ImagePlane* n_ImagePlane, Color n_BackColor, IOpenGLCanvas *n_Canvas)
{
	this->m_DataSet = n_Dataset;
	this->m_ImagePlane = n_ImagePlane;
	this->m_FrameBuffer = new unsigned char __gc[Convert::ToInt32(Math::Round(this->m_ImagePlane->Resolution)) * Convert::ToInt32(Math::Round(this->m_ImagePlane->Resolution)) * 3];
	this->m_BackColor = n_BackColor;
	this->m_Canvas = n_Canvas;
	this->m_GlobalRenderer = NULL;
	this->m_GUIHandler = NULL;
	this->m_ProgressWin = NULL;

	this->m_GradientEstimationMode = GradientEstimationSW::BY_DIFFERENCES;
	this->m_DensityInterpolationMode = InterpolationSW::BY_TRILINIEAR;
	this->m_GradientInterpolationMode = InterpolationSW::BY_TRILINIEAR;
	this->m_VoxelIDInterpolationMode = InterpolationSW::BY_NEARESTNEIGHBOUR;
	this->m_SamplingDistance = 1.0;
	this->m_ThresholdOpacity = 1.0;
	this->m_ThresholdGradient = 0.01;
	this->m_IsTwoLevel = false;
}

//Standard destructor.
RenderMachineSW::~RenderMachineSW(void)
{
	this->m_FrameBuffer = NULL;
	this->m_GlobalRenderer = NULL;
	this->m_GUIHandler = NULL;
	this->m_ProgressWin = NULL;
}

//Starts the rendering process.
void RenderMachineSW::start_Raycasting(void)
{
	if(this->m_IsTwoLevel && !this->m_IsImportance)
	{	
		do_TwoLevelRayCasting();
	}
	else if(this->m_IsTwoLevel && this->m_IsImportance)
	{
		//generate the footpint object
		Footprint *footprint = generateFootprint();

		if (footprint)
		{
			do_ImportanceDrivenRayCasting(footprint);
		}
	}
	else if(!this->m_IsTwoLevel && !this->m_IsImportance)
	{
		do_NormalRayCasting();
	}

	try {
		if (this->m_ProgressWin->isWindowActive())
		{
			this->m_ProgressWin->Close();
			try {
			this->m_Canvas->drawImage(this->m_FrameBuffer);
			}catch(Exception*) {}
		}
	}catch(Exception*) {}

	if (this->m_GUIHandler)
		this->m_GUIHandler->Start();

	//Call the garbage collector to clean up any memory leaks
	GC::Collect();
	GC::WaitForPendingFinalizers();
}

//Does normal raycasting.
void RenderMachineSW::do_NormalRayCasting(void)
{
	bool hasRayEarlyTerminated = false, isProgressbarActive = true;

	double steps_Horizontal = this->m_ImagePlane->get_Width() / this->m_ImagePlane->get_RayDistanceWidth();
	double steps_Vertical = this->m_ImagePlane->get_Heigth() / this->m_ImagePlane->get_RayDistanceHeigth();
	unsigned short numOfSamples = 0, x_down, x_up, y_down, y_up, z_down, z_up;

	float backColor __gc[] = {this->m_BackColor.get_R() / 255.0f, this->m_BackColor.get_G() / 255.0f, this->m_BackColor.get_B() / 255.0f};
	int bufferIndex = 0;

	InterpolationSW *interpolation = new InterpolationSW();
	IGradientInterpolation *gradientInterpolation = 0;
	Vector3 *nextStartPos, *intersectionPoints __gc[];
	RaySample* samplePoint = 0;

	Decimal acutalRenderStatus, renderStatusStep;

	if (this->m_DataSet->GradientsPreCalculated)
		gradientInterpolation = new GradientInterpolationWithoutPreCalcSW();
	else
		gradientInterpolation = new GradientInterpolationWithPreCalcSW();

	if(!this->m_ProgressWin)
		isProgressbarActive = false;
	else
	{
		acutalRenderStatus = 0;
		renderStatusStep = ((float)steps_Vertical / (float)(steps_Vertical * steps_Horizontal)) * 100.0f ;
		if (renderStatusStep > 1.0)
			this->m_ProgressWin->setProgressbarStepRange(Convert::ToInt32(Math::Round(Convert::ToDouble(renderStatusStep))));			
	}

	this->m_GlobalRenderer = this->m_DataSet->getGeneralDataObject()->get_RenderModel();
	
	//Raycasting
	for(int curr_Vertical = 0; curr_Vertical < steps_Vertical; curr_Vertical++)
	{
		for(int curr_Horizontal = 0; curr_Horizontal < steps_Horizontal; curr_Horizontal++)
		{
			nextStartPos = Vector3::Addition(this->m_ImagePlane->getCurrentLeftBottomPoint(),this->m_ImagePlane->getHorizontalViewVector()->multSkalar((double)curr_Horizontal + 0.5));
			nextStartPos = Vector3::Addition(nextStartPos, this->m_ImagePlane->getVerticalViewVector()->multSkalar((double)curr_Vertical + 0.5));
			intersectionPoints = calculate_IntersectionPoints(nextStartPos);
			
			//if the actual ray does not intersect the volume data -> pixel gets black color (back ground)
			if(intersectionPoints[ENTRYPOINT] == NULL && intersectionPoints[EXITPOINT] == NULL)
			{
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_R();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_G();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_B();
				continue;
			}
			//calculate the maximum count of sample point calculations
			numOfSamples = Math::Abs(Convert::ToInt16(Math::Floor((Vector3::Subtraction(intersectionPoints[EXITPOINT], intersectionPoints[ENTRYPOINT])->magnitude()) / this->m_SamplingDistance)));
			
			//initiate the global intesity and opacity of the actual ray with zero 
			this->m_GlobalRenderer->reset_Compositor();
			
			//set the Mip Renderer's flag for Local Maximum to zero
			if (this->m_GlobalRenderer->getCompositor()->GetType() == __typeof (LMipCompositingModelSW))
				__try_cast<LMipCompositingModelSW*>(this->m_GlobalRenderer->getCompositor())->set_LocalMaximumFound(false);

			//calculate the first sample point with an offset of sqrt(2)
			samplePoint = new RaySample(intersectionPoints[ENTRYPOINT]);
			//for all samples
			for(int i = 0; i < numOfSamples; i++)
			{
				//Check early ray termination with opacity thresholds
				if (this->m_GlobalRenderer->getOpacity() > this->m_ThresholdOpacity)
				{
					hasRayEarlyTerminated = true;
					break;
				}

				//Check early ray termination for mip renderer
				if (this->m_GlobalRenderer->getCompositor()->GetType() == __typeof(LMipCompositingModelSW))
					if(__try_cast<LMipCompositingModelSW*>(this->m_GlobalRenderer->getCompositor())->get_LocalMaximumFound())
						break;

				//calculate the coordinates of the neighbourhood voxels
				x_down = Convert::ToUInt16(Math::Floor(samplePoint->get_Coords()->X));
	   			x_up = Convert::ToUInt16(Math::Ceiling(samplePoint->get_Coords()->X)); 
				y_down = Convert::ToUInt16(Math::Floor(samplePoint->get_Coords()->Y)); 
				y_up = Convert::ToUInt16(Math::Ceiling(samplePoint->get_Coords()->Y)); 
				z_down = Convert::ToUInt16(Math::Floor(samplePoint->get_Coords()->Z)); 
				z_up = Convert::ToUInt16(Math::Ceiling(samplePoint->get_Coords()->Z));
			
				//check if sample point is exactly on a border of two voxels
				if (x_down == x_up) x_up++;
				if (y_down == y_up) y_up++;
				if (z_down == z_up) z_up++;

				//do trilinear interolation of density
				samplePoint->set_Density(interpolation->interpolate_Density(this->m_DensityInterpolationMode, samplePoint, this->m_DataSet, x_down, x_up, y_down, y_up, z_down, z_up));
				
				//calculate the sample points's opacity
				samplePoint = this->m_GlobalRenderer->doClassification(samplePoint);

				if (samplePoint->get_Opacity() > 0.0) 
				{	
					//only with Phong, Contour and Tone Shading the interpolated gradient of the sample is needed
					if (this->m_GlobalRenderer->need_GradientEstimation())
					{
						samplePoint->set_Gradient(gradientInterpolation->interpolateGradient(this->m_GradientInterpolationMode, this->m_GradientEstimationMode, samplePoint, this->m_DataSet,  x_down, x_up, y_down, y_up, z_down, z_up));			
						if(samplePoint->get_Gradient()->get_ScaledLength() < this->m_ThresholdGradient)
						{
							//calculate the next sample point along the ray direction
							samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
							continue;
						}
					}

					//if opacity of sample point is zero then skip global rendering
					samplePoint = this->m_GlobalRenderer->doShading(samplePoint);
					this->m_GlobalRenderer->doCompositing(samplePoint);	
				}

				//calculate the next sample point along the ray direction
				samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
			}
			
			if (!hasRayEarlyTerminated)//&& this->m_GlobalRenderer->getCompositor()->GetType() == __typeof( DvrCompositingModelSW ))
			{
				this->m_GlobalRenderer->doCompositingBackground(backColor);
				hasRayEarlyTerminated = false;
			}
		
			//save the rendered color intensity
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[R] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[G] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[B] * 255.0f));
		}

		//Set the renderstatus to active progressbar
		if(isProgressbarActive)
		{
			acutalRenderStatus = acutalRenderStatus + renderStatusStep;
			if (acutalRenderStatus > 1.0f)
			{
				try
				{
					if (!this->m_ProgressWin->isWindowActive())
						throw new Exception();

					this->m_ProgressWin->setProgressbarStep();
					acutalRenderStatus = acutalRenderStatus % 1.0f;
				}
				catch(Exception*)
				{ 
					isProgressbarActive = false; 
					this->m_FrameBuffer = NULL;
					return;
				}
			}
		}
	}
	if (isProgressbarActive)
		this->m_ProgressWin->setProgressbarToMaximum();
	nextStartPos = 0;
	intersectionPoints = 0;
	samplePoint = 0;
	interpolation = 0;
	gradientInterpolation = 0;
	acutalRenderStatus = renderStatusStep = 0;

}

//Does two-level raycasting.
void RenderMachineSW::do_TwoLevelRayCasting(void)
{
	bool hasRayEarlyTerminated = false, isProgressbarActive = true;

	double steps_Horizontal = this->m_ImagePlane->get_Width() / this->m_ImagePlane->get_RayDistanceWidth();
	double steps_Vertical = this->m_ImagePlane->get_Heigth() / this->m_ImagePlane->get_RayDistanceHeigth();
	unsigned short numOfSamples = 0, x_down, x_up, y_down, y_up, z_down, z_up, usedObjectIDs __gc[];

	float backColor __gc[] = {this->m_BackColor.get_R() / 255.0f, this->m_BackColor.get_G() / 255.0f, this->m_BackColor.get_B() / 255.0f};
	int bufferIndex = 0;

	InterpolationSW *interpolation = new InterpolationSW();
	IGradientInterpolation *gradientInterpolation = 0;
	Vector3 *nextStartPos, *intersectionPoints __gc[];
	RaySample* samplePoint = 0;

	Decimal acutalRenderStatus, renderStatusStep;

	if (this->m_DataSet->GradientsPreCalculated)
		gradientInterpolation = new GradientInterpolationWithoutPreCalcSW();
	else
		gradientInterpolation = new GradientInterpolationWithPreCalcSW();

	if(!this->m_ProgressWin)
		isProgressbarActive = false;
	else
	{
		acutalRenderStatus = 0;
		renderStatusStep = ((float)steps_Vertical / (float)(steps_Vertical * steps_Horizontal)) * 100.0f ;
		if (renderStatusStep > 1.0)
			this->m_ProgressWin->setProgressbarStepRange(Convert::ToInt32(Math::Round(Convert::ToDouble(renderStatusStep))));
	}
	
	this->m_GlobalRenderer = new RendererSW(RendererSW::TF_PHONG_DVR, NULL, NULL, new DvrCompositingModelSW());
	IRenderer* localRenderer = NULL;
	Ray* sampleRay = new Ray();
	Array* twoLevelSamples = 0;

	//Raycasting
	for(int curr_Vertical = 0; curr_Vertical < steps_Vertical; curr_Vertical++)
	{
		for(int curr_Horizontal = 0; curr_Horizontal < steps_Horizontal; curr_Horizontal++)
		{
			//calculate interscetion points of the ray with the data volume
			nextStartPos = Vector3::Addition(this->m_ImagePlane->getCurrentLeftBottomPoint(),this->m_ImagePlane->getHorizontalViewVector()->multSkalar((double)curr_Horizontal + 0.5));
			nextStartPos = Vector3::Addition(nextStartPos, this->m_ImagePlane->getVerticalViewVector()->multSkalar((double)curr_Vertical + 0.5));
			intersectionPoints = this->calculate_IntersectionPoints(nextStartPos);
			
			//if the actual ray does not intersect the volume data -> pixel gets black color (back ground)
			if(intersectionPoints[ENTRYPOINT] == NULL && intersectionPoints[EXITPOINT] == NULL)
			{
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_R();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_G();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_B();
				continue;
			}
			//calculate the maximum count of sample point calculations
			numOfSamples = Math::Abs(Convert::ToInt32(Math::Floor((Vector3::Subtraction(intersectionPoints[EXITPOINT], intersectionPoints[ENTRYPOINT])->magnitude()) / this->m_SamplingDistance)));
			sampleRay->setRayCapacity(numOfSamples);
			
			//initiate the global intensity and opacity of the actual ray with zero 
			this->m_GlobalRenderer->reset_Compositor();

			//calculate the first sample point with an offset of sqrt(2)
			samplePoint = new RaySample(intersectionPoints[ENTRYPOINT]);
			for(int i = 0; i < numOfSamples; i++)
			{
				//first (local) rendering pass of the sample points along the actual ray direction

				//calculate the coordinates of the neighbourhood voxels
				x_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->X));
	   			x_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->X)); 
				y_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Y)); 
				y_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Y)); 
				z_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Z)); 
				z_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Z));

				//check if sample point is exactly on a border of two voxels
				if (x_down == x_up) x_up++;
				if (y_down == y_up) y_up++;
				if (z_down == z_up) z_up++;
				try
				{
					//interolate density of current ray sample
					samplePoint->set_Density(interpolation->interpolate_Density(this->m_DensityInterpolationMode, samplePoint, this->m_DataSet, x_down, x_up, y_down, y_up, z_down, z_up));

					//interpolate object ID of current ray sample
					samplePoint->set_ObjectID(interpolation->interpolate_ObjectID(this->m_VoxelIDInterpolationMode, samplePoint, this->m_DataSet, x_down, x_up, y_down, y_up, z_down, z_up));
				}
				catch(Exception *e)
				{
					throw new Exception(e->Message, e);
				}
				//get actual local rederer
				localRenderer = this->m_DataSet->getObjectMember(samplePoint->get_ObjectID())->get_RenderModel();

				//Check early ray termination for mip renderer of current raysample (if local renderer is a mip type)
				if (localRenderer->getCompositor()->GetType() == __typeof(LMipCompositingModelSW))
					if(__try_cast<LMipCompositingModelSW*>(localRenderer->getCompositor())->get_LocalMaximumFound())
					{
						//calculate the next sample point along the ray direction
						samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
						continue;
					}

				//calculate the sample points's opacity
				samplePoint = localRenderer->doClassification(samplePoint);
				
				//if opacity of sample oint is zero then skip local rendering
				if (samplePoint->get_Opacity() > 0.0) 
				{   
					//only with Phong, Contour and Tone Shading the interpolated gradient of the sample is needed
					if (localRenderer->need_GradientEstimation())
					{
						try 
						{
							samplePoint->set_Gradient(gradientInterpolation->interpolateGradient(this->m_GradientInterpolationMode, this->m_GradientEstimationMode, samplePoint, this->m_DataSet,  x_down, x_up, y_down, y_up, z_down, z_up));
						}
						catch(Exception *e)
						{
							throw new Exception(e->Message, e);
						}
						if(samplePoint->get_Gradient()->get_ScaledLength() < this->m_ThresholdGradient)
						{
							//calculate the next sample point along the ray direction
							samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
							continue;
						}
					}
					
					samplePoint = localRenderer->doShading(samplePoint);
					localRenderer->doCompositing(samplePoint);

					//save the actual blended color and opacity of the local rendered object in a ray object
					sampleRay->insertSample(new RaySample(samplePoint->get_Color(), samplePoint->get_Opacity(), samplePoint->get_ObjectID()));
				}
			
				//calculate the next sample point along the ray direction
				samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
			}
			
			//second (global) rendering pass of the already local rendered sample points along the actual ray direction
			twoLevelSamples = sampleRay->getRaySamples();
			if (twoLevelSamples != NULL)
			{
				for (int nextSample = 0; nextSample < twoLevelSamples->get_Count(); nextSample++)
				{
					//Check early ray termination with opacity thresholds
					if (this->m_GlobalRenderer->getOpacity() >= this->m_ThresholdOpacity)
					{
						hasRayEarlyTerminated = true;
						break;
					}

					//composite the shaded sample colors and opacities by using DVR
					this->m_GlobalRenderer->doCompositing(__try_cast<RaySample*>(twoLevelSamples->get_Item(nextSample)));
				}
			}

			if (!hasRayEarlyTerminated)//&& this->m_GlobalRenderer->getCompositor()->GetType() == __typeof( DvrCompositingModelSW ))
			{
				this->m_GlobalRenderer->doCompositingBackground(backColor);
				hasRayEarlyTerminated = false;
			}

			//save the rendered color intensity
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[R] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[G] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[B] * 255.0f));

			usedObjectIDs = sampleRay->getUsedObjectIDs();
			if (usedObjectIDs != NULL)
			{
				for (int nextKey = 0; nextKey < usedObjectIDs->Length; nextKey++)
				{
					localRenderer = this->m_DataSet->getObjectMember(usedObjectIDs[nextKey])->get_RenderModel();
					localRenderer->reset_Compositor();

					//set the Mip Renderer's flag for Local Maximum to zero
					if (localRenderer->getCompositor()->GetType() == __typeof (LMipCompositingModelSW))
						__try_cast<LMipCompositingModelSW*>(localRenderer->getCompositor())->set_LocalMaximumFound(false);
				}
				sampleRay->clearRay();
			}
		}

		//Set the renderstatus to active progressbar
		if (isProgressbarActive)
		{
			acutalRenderStatus = acutalRenderStatus + renderStatusStep;
			if (acutalRenderStatus > 1.0f)
			{
				try
				{
					if (!this->m_ProgressWin->isWindowActive())
						throw new Exception();

					this->m_ProgressWin->setProgressbarStep();
					acutalRenderStatus = acutalRenderStatus % 1.0f;
				}
				catch(Exception*)
				{ 
					isProgressbarActive = false; 
					this->m_FrameBuffer = NULL;
					return;
				}
			}
		}
	}
	if(isProgressbarActive)
		this->m_ProgressWin->setProgressbarToMaximum();
	nextStartPos = 0;
	intersectionPoints = 0;
	samplePoint = 0;
	interpolation = 0;
	gradientInterpolation = 0;
	acutalRenderStatus = renderStatusStep = 0;
	twoLevelSamples = 0;
}

//Does importance driven raycasting.
void RenderMachineSW::do_ImportanceDrivenRayCasting(Footprint *n_Footprint)
{
	bool hasRayEarlyTerminated = false, isProgressbarActive = true, entryPointModified = false;

	InterpolationSW* interpolation = new InterpolationSW();
	IGradientInterpolation *gradientInterpolation;
	GradientEstimationSW *swg = new GradientEstimationSW();

	double steps_Horizontal = this->m_ImagePlane->get_Width() / this->m_ImagePlane->get_RayDistanceWidth(),
	       steps_Vertical = this->m_ImagePlane->get_Heigth() / this->m_ImagePlane->get_RayDistanceHeigth(),
		   distanceFromImagePlane = 0, distanceOfNewStartPoint = 0, temp_Depth = 0, actualDepth = 0;
	unsigned short numOfSamples = 0, x_down, x_up, y_down, y_up, z_down, z_up, usedObjectIDs __gc[];

	float backColor __gc[] = {this->m_BackColor.get_R() / 255.0f, this->m_BackColor.get_G() / 255.0f, this->m_BackColor.get_B() / 255.0f};
	int bufferIndex = 0;

	Vector3 *nextStartPos, *intersectionPoints __gc[], *tempEntryPoint;
	RaySample* samplePoint = 0;
	Array* twoLevelSamples = 0;

	Decimal acutalRenderStatus, renderStatusStep;

	if (this->m_DataSet->GradientsPreCalculated)
		gradientInterpolation = new GradientInterpolationWithoutPreCalcSW();
	else
		gradientInterpolation = new GradientInterpolationWithPreCalcSW();

	if(!this->m_ProgressWin)
		isProgressbarActive = false;
	else
	{
		acutalRenderStatus = 0;
		renderStatusStep = ((float)steps_Vertical / (float)(steps_Vertical * steps_Horizontal)) * 100.0f ;
		if (renderStatusStep > 1.0)
			this->m_ProgressWin->setProgressbarStepRange(Convert::ToInt32(Math::Round(Convert::ToDouble(renderStatusStep))));

		//progressbar parameters
		this->m_ProgressWin->resetGUIItems(S"Rendering Process Control", S"The data is being rendered now");
	}

	this->m_GlobalRenderer = new RendererSW(RendererSW::TF_PHONG_DVR, NULL, NULL, new DvrCompositingModelSW());
	IRenderer* localRenderer = NULL;
	Ray* sampleRay = new Ray();
	this->m_IDVRCompositingModel = m_DataSet->getIIDVRCompositingModel();

	//setup the footprint object
	n_Footprint->setSlope(__try_cast<IDVRMImPCompositingModelSW*>(this->m_IDVRCompositingModel)->getCountersinkSlope());
	n_Footprint->setGradientScaleFactor(__try_cast<IDVRMImPCompositingModelSW*>(this->m_IDVRCompositingModel)->getGradientScaleFactor());
	
	//calculate the distance array to the object with maximal importance
	n_Footprint->calculateDistanceArray();

	for(int curr_Vertical = 0; curr_Vertical < steps_Vertical; curr_Vertical++)
	{
		for(int curr_Horizontal = 0; curr_Horizontal < steps_Horizontal; curr_Horizontal++)
		{
			//calculate interscetion points of the ray with the data volume
			nextStartPos = Vector3::Addition(this->m_ImagePlane->getCurrentLeftBottomPoint(),this->m_ImagePlane->getHorizontalViewVector()->multSkalar((double)curr_Horizontal + 0.5));
			nextStartPos = Vector3::Addition(nextStartPos, this->m_ImagePlane->getVerticalViewVector()->multSkalar((double)curr_Vertical + 0.5));
			intersectionPoints = this->calculate_IntersectionPoints(nextStartPos);
			
			//if the actual ray does not intersect the volume data -> pixel gets black color (back ground)
			if(intersectionPoints[ENTRYPOINT] == NULL && intersectionPoints[EXITPOINT] == NULL)
			{
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_R();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_G();
				this->m_FrameBuffer[bufferIndex++] = this->m_BackColor.get_B();
				continue;
			}
		
			//calculate new entrypoint of a ray
			entryPointModified = false;
			distanceFromImagePlane = this->m_ImagePlane->calculateDistance(intersectionPoints[ENTRYPOINT]);
			distanceOfNewStartPoint = n_Footprint->calculateDistanceOfNewStartingPoint(curr_Horizontal, curr_Vertical);

			//if both are on the same side of te image plane
			if(distanceOfNewStartPoint != Double::MinValue)
			{
				if(Math::Sign(distanceFromImagePlane) == Math::Sign(distanceOfNewStartPoint))
				{
					tempEntryPoint = Vector3::Addition(nextStartPos, Vector3::CreateNormalizedVector(this->m_ImagePlane->get_ViewDirection())->multSkalar(distanceOfNewStartPoint));

					//if new entypoint is between old entry and exit point
					if(this->m_ImagePlane->calculateDistance(tempEntryPoint) < this->m_ImagePlane->calculateDistance(intersectionPoints[EXITPOINT]) && 
					   this->m_ImagePlane->calculateDistance(tempEntryPoint) > this->m_ImagePlane->calculateDistance(intersectionPoints[ENTRYPOINT]))
					{
						//entrypoint has been modified
						intersectionPoints[ENTRYPOINT] = tempEntryPoint;
						entryPointModified = true;
					}
				}
			}
			
			//calculate the maximum count of sample point calculations
			numOfSamples = Math::Abs(Convert::ToInt32(Math::Floor((Vector3::Subtraction(intersectionPoints[EXITPOINT], intersectionPoints[ENTRYPOINT])->magnitude()) / this->m_SamplingDistance)));
			sampleRay->setRayCapacity(numOfSamples);
			
			//initiate the global intensity and opacity of the actual ray with zero 
			this->m_GlobalRenderer->reset_Compositor();

			//calculate the first sample point with an offset of sqrt(2)
			samplePoint = new RaySample(intersectionPoints[ENTRYPOINT]);

			//do importance compositing
			m_IDVRCompositingModel->setFootprintArrayList(n_Footprint->getFootprintArrayList(curr_Horizontal, curr_Vertical));
			m_IDVRCompositingModel->doCompositing();

			temp_Depth = n_Footprint->calculateDepthOfObjectWithMaxImpAlongFootprintAL(curr_Horizontal, curr_Vertical);
			
			for(int i = 0; i < numOfSamples; i++)
			{
				//first (local) rendering pass of the sample points along the actual ray direction

				//calculate the coordinates of the neighbourhood voxels
				x_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->X));
	   			x_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->X)); 
				y_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Y)); 
				y_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Y)); 
				z_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Z)); 
				z_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Z));

				//check if sample point is exactly on a border of two voxels
				if (x_down == x_up) x_up++;
				if (y_down == y_up) y_up++;
				if (z_down == z_up) z_up++;
				try
				{
					//interolate density of current ray sample
					samplePoint->set_Density(interpolation->interpolate_Density(this->m_DensityInterpolationMode, samplePoint, this->m_DataSet, x_down, x_up, y_down, y_up, z_down, z_up));

					//interpolate object ID of current ray sample
					samplePoint->set_ObjectID(interpolation->interpolate_ObjectID(this->m_VoxelIDInterpolationMode, samplePoint, this->m_DataSet, x_down, x_up, y_down, y_up, z_down, z_up));
				}
				catch(Exception *e)
				{
					throw new Exception(e->Message, e);
				}
				//get actual local rederer
				localRenderer = this->m_DataSet->getObjectMember(samplePoint->get_ObjectID())->get_RenderModel();

				//Check early ray termination for mip renderer of current raysample (if local renderer is a mip type)
				if (localRenderer->getCompositor()->GetType() == __typeof(LMipCompositingModelSW))
					if(__try_cast<LMipCompositingModelSW*>(localRenderer->getCompositor())->get_LocalMaximumFound())
					{
						//calculate the next sample point along the ray direction
						samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
						continue;
					}

				//calculate the sample points's opacity
				samplePoint = localRenderer->doClassification(samplePoint);

				//if opacity of sample oint is zero then skip local rendering
				if (samplePoint->get_Opacity() > 0.0) 
				{   
					//only with Phong, Contour and Tone Shading the interpolated gradient of the sample is needed
					if (localRenderer->need_GradientEstimation())
					{
						try 
						{
							//modify gradient only once at the modified entry points
							if(i == 0 && entryPointModified && samplePoint->get_Density() != 0)
							{
								if(this->m_GradientEstimationMode == GradientEstimationSW::BY_DIFFERENCES)
								{
									samplePoint->set_Gradient(n_Footprint->modifyGradient(curr_Horizontal, curr_Vertical, swg->get_scaleFactorDifferences(), this->m_ImagePlane->get_RotMatrix()));
								}
								if(this->m_GradientEstimationMode == GradientEstimationSW::BY_4DREGRESSION)
								{
									samplePoint->set_Gradient(n_Footprint->modifyGradient(curr_Horizontal, curr_Vertical, swg->get_scaleFactorRegression(), this->m_ImagePlane->get_RotMatrix()));
								}
							}
							else
							{
								samplePoint->set_Gradient(gradientInterpolation->interpolateGradient(this->m_GradientInterpolationMode, this->m_GradientEstimationMode, samplePoint, this->m_DataSet,  x_down, x_up, y_down, y_up, z_down, z_up));
							}
						}
						catch(Exception *e)
						{
							throw new Exception(e->Message, e);
						}
						if(samplePoint->get_Gradient()->get_ScaledLength() < this->m_ThresholdGradient && !entryPointModified)
						{
							//calculate the next sample point along the ray direction
							samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
							continue;
						}
					}
					
					samplePoint = localRenderer->doShading(samplePoint);
					
					actualDepth = this->m_ImagePlane->calculateDistance(samplePoint->get_Coords());
					if(actualDepth < temp_Depth)
					{
						samplePoint = m_IDVRCompositingModel->modifyRaySample(samplePoint);
					}
					
					localRenderer->doCompositing(samplePoint);

					//save the actual blended color and opacity of the local rendered object in a ray object
					sampleRay->insertSample(new RaySample(samplePoint->get_Color(), samplePoint->get_Opacity(), samplePoint->get_ObjectID()));
				}
			
				//calculate the next sample point along the ray direction
				samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->multSkalar(this->m_SamplingDistance)));
			}
			
			//second (global) rendering pass of the already local rendered sample points along the actual ray direction
			twoLevelSamples = sampleRay->getRaySamples();
			if (twoLevelSamples != NULL)
			{
				for (int nextSample = 0; nextSample < twoLevelSamples->get_Count(); nextSample++)
				{
					//Check early ray termination with opacity thresholds
					if (this->m_GlobalRenderer->getOpacity() >= this->m_ThresholdOpacity)
					{
						hasRayEarlyTerminated = true;
						break;
					}

					//composite the shaded sample colors and opacities by using DVR
					this->m_GlobalRenderer->doCompositing(__try_cast<RaySample*>(twoLevelSamples->get_Item(nextSample)));
				}
			}

			if (!hasRayEarlyTerminated)//&& this->m_GlobalRenderer->getCompositor()->GetType() == __typeof( DvrCompositingModelSW ))
			{
				this->m_GlobalRenderer->doCompositingBackground(backColor);
				hasRayEarlyTerminated = false;
			}

			//save the rendered color intensity
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[R] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[G] * 255.0f));
			this->m_FrameBuffer[bufferIndex++] = 
										Convert::ToByte(Math::Round(this->m_GlobalRenderer->getIntensity()[B] * 255.0f));


			usedObjectIDs = sampleRay->getUsedObjectIDs();
			if (usedObjectIDs != NULL)
			{
				for (int nextKey = 0; nextKey < usedObjectIDs->Length; nextKey++)
				{
					localRenderer = this->m_DataSet->getObjectMember(usedObjectIDs[nextKey])->get_RenderModel();
					localRenderer->reset_Compositor();

					//set the Mip Renderer's flag for Local Maximum to zero
					if (localRenderer->getCompositor()->GetType() == __typeof (LMipCompositingModelSW))
						__try_cast<LMipCompositingModelSW*>(localRenderer->getCompositor())->set_LocalMaximumFound(false);
				}
				sampleRay->clearRay();
			}
		}

		//Set the renderstatus to active progressbar
		if (isProgressbarActive)
		{
			acutalRenderStatus = acutalRenderStatus + renderStatusStep;
			if (acutalRenderStatus > 1.0f)
			{
				try
				{
					if (!this->m_ProgressWin->isWindowActive())
						throw new Exception();

					this->m_ProgressWin->setProgressbarStep();
					acutalRenderStatus = acutalRenderStatus % 1.0f;
				}
				catch(Exception*)
				{ 
					isProgressbarActive = false; 
					this->m_FrameBuffer = NULL;
					return;
				}
			}
		}
	}
	if (isProgressbarActive)
		this->m_ProgressWin->setProgressbarToMaximum();
	nextStartPos = 0;
	intersectionPoints = 0;
	samplePoint = 0;
	interpolation = 0;
	gradientInterpolation = 0;
	acutalRenderStatus = renderStatusStep = 0;
	twoLevelSamples = 0;
	swg = 0;
}

//Sets the volume dataset.
void RenderMachineSW::set_Data(VolumeData* n_DataSet)
{
	this->m_DataSet = n_DataSet;
}

//Sets the two-level rendering flag.
void RenderMachineSW::set_TwoLevelRendering(bool n_IsTwoLevel)
{
	this->m_IsTwoLevel = n_IsTwoLevel;
}

//Sets the importance driven rendering flag.
void RenderMachineSW::set_ImportanceDrivenRendering(bool n_IsImportance)
{
	this->m_IsImportance = n_IsImportance;
}

//Sets the sampling distance.
void RenderMachineSW::set_SampingDistance(double n_SamplingDistance)
{
	this->m_SamplingDistance = n_SamplingDistance;
}

//Sets the openGL canvas.
void RenderMachineSW::setOpenGLCanvas(IOpenGLCanvas *n_Canvas)
{
	this->m_Canvas = n_Canvas;
}

//Sets the opacity threshold.
void RenderMachineSW::set_OpacityThreshold(double n_ThresholdOpacity)
{
	this->m_ThresholdOpacity = n_ThresholdOpacity;
}

//Sets the gradient threshold.
void RenderMachineSW::set_GradientThreshold(double n_ThresholdGradient)
{
	this->m_ThresholdGradient = n_ThresholdGradient;
}

//Sets the image plane.
void RenderMachineSW::set_ImagePlane(RenderPrimitives::ImagePlane* n_ImagePlane)
{
	this->m_ImagePlane = n_ImagePlane;
}

//Sets the gradient estimation mode.
void RenderMachineSW::set_GradientEstimationMode(char n_GradientEstimationMode)
{
	this->m_GradientEstimationMode = n_GradientEstimationMode;
}

//Sets the density interpolation mode.
void RenderMachineSW::set_DensityInterpolationMode(char n_DensityInterpolationMode)
{
	this->m_DensityInterpolationMode = n_DensityInterpolationMode;
}

//Sets the gradient interpolation mode.
void RenderMachineSW::set_GradientInterpolationMode(char n_GradientInterpolationMode)
{
	this->m_GradientEstimationMode = n_GradientInterpolationMode;
}

//Sets the voxelID interpolation mode.
void RenderMachineSW::set_VoxelIDInterpolationMode(char n_VoxelIDInterpolationMode)
{
	this->m_VoxelIDInterpolationMode = n_VoxelIDInterpolationMode;
}

//Returns the image plane.
RenderPrimitives::ImagePlane* RenderMachineSW::get_ImagePlane(void)
{
	return this->m_ImagePlane;
}

//Returns the volume dataset.
VolumeData* RenderMachineSW::get_Data(void)
{
	return this->m_DataSet;
}

//Sets the progress bar.
void RenderMachineSW::setProgressWin(IDVR::ProgressWin *n_ProgressWin)
{
	this->m_ProgressWin = n_ProgressWin;
}

//Sets the GUI handler.
void RenderMachineSW::setGUIHandler(Thread *n_GUIHandler)
{
	this->m_GUIHandler = n_GUIHandler;	
}

//Returns the framebuffer.
unsigned char RenderMachineSW::getFrameBuffer(void) __gc[]
{
	return this->m_FrameBuffer;
}

//Calculates the intersection points of a ray with the volume.
Vector3* RenderMachineSW::calculate_IntersectionPoints(Vector3* startPoint) __gc[]
{
	double lambda = -1.0; bool haveIntersectionPoint = true;
	ArrayList* intersectionPoints = new ArrayList(6);
	Vector3* intersectionPointsCoords __gc[] = new Vector3*[2]; 
	short dimension __gc[] = this->m_DataSet->get_VolumeDimension();
	double A = 0.0, B = 0.0, C = 0.0, D = 0.0;

	Vector3 *view = Vector3::CreateNormalizedVector(this->m_ImagePlane->get_ViewDirection());
	
	//Left plane
	A = 1.0, B = 0.0, C = 0.0, D = -1.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->Y >= 1.0 && intersection->Y <= dimension[1] - 2) && (intersection->Z >= 1.0 && intersection->Z <= dimension[2] - 2))
			intersectionPoints->Add(intersection);
	}
	
	//Right plane
	A = 1.0, B = 0.0, C = 0.0, D = -dimension[X] + 2.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->Y >= 1.0 && intersection->Y <= dimension[1] - 2) && (intersection->Z >= 1.0 && intersection->Z <= dimension[2] - 2))
			intersectionPoints->Add(intersection);
	}

	//Top plane
	A = 0.0, B = 1.0, C = 0.0, D = -1.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= dimension[0] - 2) && (intersection->Z >= 1.0 && intersection->Z <= dimension[2] - 2))
			intersectionPoints->Add(intersection);
	}

	//Bottom plane
	A = 0.0, B = 1.0, C = 0.0, D = -dimension[Y] + 2.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= dimension[0] - 2) && (intersection->Z >= 1.0 && intersection->Z <= dimension[2] - 2))
			intersectionPoints->Add(intersection);
	}

	//Front plane
	A = 0.0, B = 0.0, C = 1.0, D = -1.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= dimension[0] - 2) && (intersection->Y >= 1.0 && intersection->Y <= dimension[1] - 2))
			intersectionPoints->Add(intersection);
	}

	//Back plan
	A = 0.0, B = 0.0, C = 1.0, D = -dimension[Z] + 2.0;

	if((A * view->X + B * view->Y + C * view->Z) != 0)
	{
		lambda = (-(A * startPoint->X + B * startPoint->Y + C * startPoint->Z + D)) /
				 (A * view->X + B * view->Y + C * view->Z);
	}
	if(lambda >= 0.0)
	{
		Vector3* intersection = Vector3::Addition(startPoint, view->multSkalar(lambda));
		if ((intersection->X >= 1.0 && intersection->X <= dimension[0] - 2) && (intersection->Y >= 1.0 && intersection->Y <= dimension[1] - 2))
			intersectionPoints->Add(intersection);
	}

	//Check which is entry point and exit point
	if(intersectionPoints->get_Count() > 1)
	{
		double distance __gc[] = new double __gc[intersectionPoints->get_Count()];

		double maximalDistance = Double::MinValue;
		double minimalDistance = Double::MaxValue;
		int minimalIndex = -1;
		int maximalIndex = -1;
		
		for(int elements = 0; elements < distance->get_Count(); elements++)
		{
			distance[elements] = Vector3::Subtraction(__try_cast<Vector3*>(intersectionPoints->get_Item(elements)), startPoint)->magnitude();

			if(distance[elements] < minimalDistance)
			{
				minimalDistance = distance[elements];
				minimalIndex = elements;
			}
			if(distance[elements] > maximalDistance)
			{
				maximalDistance = distance[elements];
				maximalIndex = elements;
			}
		}

		intersectionPointsCoords[0] =  __try_cast<Vector3*>(intersectionPoints->get_Item(minimalIndex));
		intersectionPointsCoords[1] =  __try_cast<Vector3*>(intersectionPoints->get_Item(maximalIndex));

		/*double distance_1 = Vector3::Subtraction(__try_cast<Vector3*>(intersectionPoints->get_Item(0)), startPoint)->magnitude();
		double distance_2 = Vector3::Subtraction(__try_cast<Vector3*>(intersectionPoints->get_Item(intersectionPoints->get_Count() - 1)), startPoint)->magnitude();

		if (distance_1 <= distance_2)
		{
			intersectionPointsCoords[0] = __try_cast<Vector3*>(intersectionPoints->get_Item(0));
			intersectionPointsCoords[1] = __try_cast<Vector3*>(intersectionPoints->get_Item(intersectionPoints->get_Count() - 1));
		}
		else
		{
			intersectionPointsCoords[0] = __try_cast<Vector3*>(intersectionPoints->get_Item(intersectionPoints->get_Count() - 1));
			intersectionPointsCoords[1] = __try_cast<Vector3*>(intersectionPoints->get_Item(0));
		}*/
	}
	else
	{
		intersectionPointsCoords[0] = intersectionPointsCoords[1] = NULL;
	}

	intersectionPoints = 0;
	return intersectionPointsCoords;
}

//Generates the footprint object.
Footprint* RenderMachineSW::generateFootprint(void)
{
	bool isProgressbarActive = true;

	double steps_Horizontal = this->m_ImagePlane->get_Width() / this->m_ImagePlane->get_RayDistanceWidth();
	double steps_Vertical = this->m_ImagePlane->get_Heigth() / this->m_ImagePlane->get_RayDistanceHeigth();
	unsigned short numOfSamples = 0;

	Vector3 *nextStartPos, *intersectionPoints __gc[];
 
	Decimal acutalGenerationStatus, generationStatusStep;

	if(!this->m_ProgressWin)
		isProgressbarActive = false;
	else
	{
		acutalGenerationStatus = 0;
		generationStatusStep = ((float)steps_Vertical / (float)(steps_Vertical * steps_Horizontal)) * 100.0f ;
		if (generationStatusStep > 1.0)
			this->m_ProgressWin->setProgressbarStepRange(Convert::ToInt32(Math::Round(Convert::ToDouble(generationStatusStep))));
	}
	
	Footprint *footprint = new Footprint(Convert::ToInt32(Math::Round(steps_Horizontal)), Convert::ToInt32(Math::Round(steps_Vertical)), this->m_ImagePlane, this->m_SamplingDistance, this->m_DataSet);
	
	//generate the footprint object
	for(int curr_Vertical = 0; curr_Vertical < steps_Vertical; curr_Vertical++)
	{
		for(int curr_Horizontal = 0; curr_Horizontal < steps_Horizontal; curr_Horizontal++)
		{
			nextStartPos = Vector3::Addition(this->m_ImagePlane->getCurrentLeftBottomPoint(),this->m_ImagePlane->getHorizontalViewVector()->multSkalar((double)curr_Horizontal + 0.5));
			nextStartPos = Vector3::Addition(nextStartPos, this->m_ImagePlane->getVerticalViewVector()->multSkalar((double)curr_Vertical + 0.5));
			intersectionPoints = this->calculate_IntersectionPoints(nextStartPos);
			
			if(intersectionPoints[ENTRYPOINT] == NULL && intersectionPoints[EXITPOINT] == NULL) continue;
			
			numOfSamples = Math::Abs(Convert::ToInt32(Math::Floor((Vector3::Subtraction(intersectionPoints[EXITPOINT], intersectionPoints[ENTRYPOINT])->magnitude()) / this->m_SamplingDistance)));
			
			footprint->createFootprint(curr_Horizontal, curr_Vertical, intersectionPoints[EXITPOINT], numOfSamples, InterpolationSW::BY_NEARESTNEIGHBOUR);
		}

		//Set the renderstatus to active progressbar
		if (isProgressbarActive)
		{
			acutalGenerationStatus = acutalGenerationStatus + generationStatusStep;
			if (acutalGenerationStatus > 1.0f)
			{
				try
				{
					if (!this->m_ProgressWin->isWindowActive())
						throw new Exception();

					this->m_ProgressWin->setProgressbarStep();
					acutalGenerationStatus = acutalGenerationStatus % 1.0f;
				}
				catch(Exception*)
				{ 
					isProgressbarActive = false; 
					footprint = NULL;
					return NULL;
				}
			}	
		}
	}
	if (isProgressbarActive)
		this->m_ProgressWin->setProgressbarToMaximum();
	
	nextStartPos = 0; 
	intersectionPoints = 0;
	acutalGenerationStatus = generationStatusStep = 0;

	return footprint;
}