#include "StdAfx.h"
#include ".\renderersw.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::SWVolumeRenderer;

//Standard constructor.
RendererSW::RendererSW(void)
{
	this->m_IClassificator = NULL;
	this->m_IShader = NULL;
	this->m_ICompositor = NULL;
	this->m_RenderMode = -1;
}

//Constructor.
RendererSW::RendererSW(char n_RenderMode, IClassificationModel *n_IClassificator, IShadingModel *n_IShader, ICompositingModel *n_ICompositor)
{
	this->m_RenderMode = n_RenderMode;
	this->m_IClassificator = n_IClassificator;
	this->m_IShader = n_IShader;
	this->m_ICompositor = n_ICompositor;
}

//Standard destructor.
RendererSW::~RendererSW(void)
{
	this->m_IClassificator = NULL;
	this->m_IShader = NULL;
	this->m_ICompositor = NULL;
}

//Calculates the opacity value for a given raysample.
RaySample* RendererSW::doClassification(RaySample *n_RaySample)
{
	return this->m_IClassificator->doClassification(n_RaySample);
}

//Calculates the intensity value for a given raysample.
RaySample* RendererSW::doShading(RaySample *n_RaySample)
{
	return this->m_IShader->doShading(n_RaySample);
}

//Blends the final intensity and opacity values of a given raysample.
void RendererSW::doCompositing(RaySample *n_RaySample)
{
	this->m_ICompositor->doCompositing(n_RaySample);
}

//Blends the final pixelcolor with the given background color.
void RendererSW::doCompositingBackground(float n_BackColor __gc[])
{
	this->m_ICompositor->doCompositingBackground(n_BackColor);
}

//Sets the classification model.
 void RendererSW::setClassificator(IClassificationModel *n_IClassificator)
 {
	 this->m_IClassificator = n_IClassificator;
 }

//Sets the shading model.
void RendererSW::setShader(IShadingModel *n_IShader)
{
	this->m_IShader = n_IShader;
}

//Sets the composition model.
void RendererSW::setCompositor(ICompositingModel *n_ICompositor)
{
	this->m_ICompositor = n_ICompositor;
}

//Returns the classification model.
IClassificationModel* RendererSW::getClassificator(void)
{
	return this->m_IClassificator;
}

//Returns the shading model.
IShadingModel* RendererSW::getShader(void)
{
	return this->m_IShader;
}

//Returns the composition model.
ICompositingModel* RendererSW::getCompositor(void)
{
	return this->m_ICompositor;
}

//Returns true if gradinet estimation is needed.
bool RendererSW::need_GradientEstimation(void)
{
	if (this->m_RenderMode == LIN_MIP_LMIP || this->m_RenderMode == TF_MIP_LMIP)
		return false;

	return true;
}

//Resets the values of the composition model.
void RendererSW::reset_Compositor(void)
{
	this->m_ICompositor->resetValues();
}

//Returns the current intensity array.
float RendererSW::getIntensity(void) __gc[]
{
	return this->m_ICompositor->get_Intensity();
}

//Returns the current opacity value.
float RendererSW::getOpacity(void)
{
	return this->m_ICompositor->get_Opacity();
}

//Sets the render mode.
void RendererSW::setRenderMode(char n_RenderMode)
{
	this->m_RenderMode = n_RenderMode;
}

//Returns the render mode.
char RendererSW::getRenderMode(void)
{
	return this->m_RenderMode;
}