#pragma once

#include "VolumeData.h"
#include "ClearCanvasGL.h"
#include "LinearClassificationModelSW.h"
#include "MipShadingModelSW.h"
#include "DrawModelGLSW.h"
#include "DataLoader.h"
#include "RaySample.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

using namespace IDVR::Volume;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::Tools::OpenGLTools;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

namespace IDVR
{
	/**
	<B>GUI class for the slice viewer.</B>

	namespace: IDVR

	Baseclass: System::Windows::Forms::Form

	This is the GUI class for the slice viewer. It provides methods to view the whole volume dataset in slices
	perpendicular to the main axes.

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-20-2005
	*/
	public __gc class SliceViewer : public System::Windows::Forms::Form
	{
	public: 

		/**
		Constructor.
		@param n_VolumeData the volume dataset.
		@param n_VolumeDataDescription the path and name of the volume dataset.
		*/
		SliceViewer(VolumeData *n_VolumeData, String *n_VolumeDataDescription)
		{
			InitializeComponent();

			this->Text = String::Concat(MAINTITLE, S" - [loaded dataset] ", n_VolumeDataDescription);
			this->m_SubItem_SliceViewer = this->m_TbSubItem_SliceViewer = NULL;
			this->m_VolumeData = n_VolumeData;
			this->cob_SliceXYZ->set_SelectedIndex(2);
			this->m_ClearCanvas = new ClearCanvasGL();
			this->openGLPanel->set_Code(this->m_ClearCanvas);
			this->pan_SliceColor->set_BackColor(this->colDia_SliceColor->get_Color());
			this->tex_SliceNumber->set_Text(Convert::ToString(this->trb_Slices->get_Value()));
			//this->m_SamplingFactor = 1;
			this->cob_SliceSize->set_SelectedIndex(0);
		}

		/**
		Sets the subitem for the GUI.
		@param n_SubItem_SliceViewer the subitem for the GUI.
		*/
		void setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_SliceViewer);
		/**
		Sets the toolbar subitem for the GUI.
		@param n_TbSubItem_SliceViewer the toolbar subitem for the GUI.
		*/
		void setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_SliceViewer);

		/**
		Resizes the canvas.
		*/
		void resizeCanvas(void);
		/**
		Draws the actual slice.
		*/
		void drawSlice(void);
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
	private: System::ComponentModel::IContainer *  components;

	private:

		TD::SandBar::MenuButtonItem *m_SubItem_SliceViewer; /**< the subitem for the GUI. */
		TD::SandBar::MenuButtonItem *m_TbSubItem_SliceViewer; /**< the toolbar subitem for the GUI. */
		VolumeData *m_VolumeData; /**< The volume dataset. */
		ClearCanvasGL *m_ClearCanvas; /**< Object to clear the canvas. */

		LinearClassificationModelSW *m_Classificator; /**< Linear classificator. */
		MipShadingModelSW *m_Shader; /**< Mip shading model. */

		DrawModelGLSW *m_DrawModel; /**< Object to draw on the canvas. */

		short m_SamplingFactor; /**< Sampling factor. */

		static String *MAINTITLE = S"Slice Viewer"; /**< Title of the GUI. */

		__value enum Indices{X = 0, Y, Z};

	private: System::Windows::Forms::GroupBox *  grb_SliceControl;
	private: System::Windows::Forms::ComboBox *  cob_SliceXYZ;
	private: System::Windows::Forms::TrackBar *  trb_Slices;
	private: System::Windows::Forms::Panel *  pan_Slices;
	private: System::Windows::Forms::Button *  but_SliceColor;

	private: System::Windows::Forms::ColorDialog *  colDia_SliceColor;
	private: System::Windows::Forms::Panel *  pan_SliceColor;
	private: System::Windows::Forms::Label *  lab_SliceNumber;
	private: System::Windows::Forms::TextBox *  tex_SliceNumber;

	private: System::Windows::Forms::ToolTip *  toolTip1;
	private: System::Windows::Forms::ImageList *  iml_ButtonIcon;
	private: System::Windows::Forms::Label *  lab_SliceSize;
	private: System::Windows::Forms::ComboBox *  cob_SliceSize;


	private: OpenGLPanel::OpenGLPanel *  openGLPanel;

		/**
		Initializes the GUI components
		*/
		void InitializeComponent(void)
		{
			this->components = new System::ComponentModel::Container();
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(IDVR::SliceViewer));
			this->grb_SliceControl = new System::Windows::Forms::GroupBox();
			this->cob_SliceSize = new System::Windows::Forms::ComboBox();
			this->lab_SliceSize = new System::Windows::Forms::Label();
			this->tex_SliceNumber = new System::Windows::Forms::TextBox();
			this->lab_SliceNumber = new System::Windows::Forms::Label();
			this->pan_SliceColor = new System::Windows::Forms::Panel();
			this->but_SliceColor = new System::Windows::Forms::Button();
			this->iml_ButtonIcon = new System::Windows::Forms::ImageList(this->components);
			this->cob_SliceXYZ = new System::Windows::Forms::ComboBox();
			this->trb_Slices = new System::Windows::Forms::TrackBar();
			this->pan_Slices = new System::Windows::Forms::Panel();
			this->openGLPanel = new OpenGLPanel::OpenGLPanel();
			this->colDia_SliceColor = new System::Windows::Forms::ColorDialog();
			this->toolTip1 = new System::Windows::Forms::ToolTip(this->components);
			this->grb_SliceControl->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_Slices))->BeginInit();
			this->pan_Slices->SuspendLayout();
			this->SuspendLayout();
			// 
			// grb_SliceControl
			// 
			this->grb_SliceControl->Controls->Add(this->cob_SliceSize);
			this->grb_SliceControl->Controls->Add(this->lab_SliceSize);
			this->grb_SliceControl->Controls->Add(this->tex_SliceNumber);
			this->grb_SliceControl->Controls->Add(this->lab_SliceNumber);
			this->grb_SliceControl->Controls->Add(this->pan_SliceColor);
			this->grb_SliceControl->Controls->Add(this->but_SliceColor);
			this->grb_SliceControl->Controls->Add(this->cob_SliceXYZ);
			this->grb_SliceControl->Controls->Add(this->trb_Slices);
			this->grb_SliceControl->Location = System::Drawing::Point(8, 8);
			this->grb_SliceControl->Name = S"grb_SliceControl";
			this->grb_SliceControl->Size = System::Drawing::Size(600, 56);
			this->grb_SliceControl->TabIndex = 0;
			this->grb_SliceControl->TabStop = false;
			this->grb_SliceControl->Text = S"Slice Control";
			// 
			// cob_SliceSize
			// 
			System::Object* __mcTemp__1[] = new System::Object*[8];
			__mcTemp__1[0] = S"Original";
			__mcTemp__1[1] = S"2x";
			__mcTemp__1[2] = S"3x";
			__mcTemp__1[3] = S"4x";
			__mcTemp__1[4] = S"5x";
			__mcTemp__1[5] = S"6x";
			__mcTemp__1[6] = S"7x";
			__mcTemp__1[7] = S"8x";
			this->cob_SliceSize->Items->AddRange(__mcTemp__1);
			this->cob_SliceSize->Location = System::Drawing::Point(220, 20);
			this->cob_SliceSize->Name = S"cob_SliceSize";
			this->cob_SliceSize->Size = System::Drawing::Size(64, 21);
			this->cob_SliceSize->TabIndex = 7;
			this->cob_SliceSize->SelectedIndexChanged += new System::EventHandler(this, perform_ActionComboBoxSelectedIndexChanged);
			// 
			// lab_SliceSize
			// 
			this->lab_SliceSize->Location = System::Drawing::Point(180, 20);
			this->lab_SliceSize->Name = S"lab_SliceSize";
			this->lab_SliceSize->Size = System::Drawing::Size(32, 23);
			this->lab_SliceSize->TabIndex = 6;
			this->lab_SliceSize->Text = S"Size:";
			this->lab_SliceSize->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// tex_SliceNumber
			// 
			this->tex_SliceNumber->Location = System::Drawing::Point(548, 20);
			this->tex_SliceNumber->Name = S"tex_SliceNumber";
			this->tex_SliceNumber->Size = System::Drawing::Size(44, 20);
			this->tex_SliceNumber->TabIndex = 5;
			this->tex_SliceNumber->Text = S"";
			this->tex_SliceNumber->KeyDown += new System::Windows::Forms::KeyEventHandler(this, perform_ActionTextboxKeydown);
			// 
			// lab_SliceNumber
			// 
			this->lab_SliceNumber->Location = System::Drawing::Point(468, 20);
			this->lab_SliceNumber->Name = S"lab_SliceNumber";
			this->lab_SliceNumber->Size = System::Drawing::Size(76, 23);
			this->lab_SliceNumber->TabIndex = 4;
			this->lab_SliceNumber->Text = S"Slice Number:";
			this->lab_SliceNumber->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// pan_SliceColor
			// 
			this->pan_SliceColor->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->pan_SliceColor->Location = System::Drawing::Point(148, 20);
			this->pan_SliceColor->Name = S"pan_SliceColor";
			this->pan_SliceColor->Size = System::Drawing::Size(24, 21);
			this->pan_SliceColor->TabIndex = 3;
			// 
			// but_SliceColor
			// 
			this->but_SliceColor->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_SliceColor->ImageAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->but_SliceColor->ImageIndex = 0;
			this->but_SliceColor->ImageList = this->iml_ButtonIcon;
			this->but_SliceColor->Location = System::Drawing::Point(85, 20);
			this->but_SliceColor->Name = S"but_SliceColor";
			this->but_SliceColor->Size = System::Drawing::Size(56, 21);
			this->but_SliceColor->TabIndex = 2;
			this->but_SliceColor->Text = S"Color";
			this->but_SliceColor->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			this->but_SliceColor->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// iml_ButtonIcon
			// 
			this->iml_ButtonIcon->ImageSize = System::Drawing::Size(16, 16);
			this->iml_ButtonIcon->ImageStream = (__try_cast<System::Windows::Forms::ImageListStreamer *  >(resources->GetObject(S"iml_ButtonIcon.ImageStream")));
			this->iml_ButtonIcon->TransparentColor = System::Drawing::Color::Transparent;
			// 
			// cob_SliceXYZ
			// 
			System::Object* __mcTemp__2[] = new System::Object*[3];
			__mcTemp__2[0] = S"X - Slices";
			__mcTemp__2[1] = S"Y - Slices";
			__mcTemp__2[2] = S"Z - Slices";
			this->cob_SliceXYZ->Items->AddRange(__mcTemp__2);
			this->cob_SliceXYZ->Location = System::Drawing::Point(8, 20);
			this->cob_SliceXYZ->Name = S"cob_SliceXYZ";
			this->cob_SliceXYZ->Size = System::Drawing::Size(72, 21);
			this->cob_SliceXYZ->TabIndex = 0;
			this->cob_SliceXYZ->SelectedIndexChanged += new System::EventHandler(this, perform_ActionComboBoxSelectedIndexChanged);
			// 
			// trb_Slices
			// 
			this->trb_Slices->AutoSize = false;
			this->trb_Slices->Location = System::Drawing::Point(292, 16);
			this->trb_Slices->Name = S"trb_Slices";
			this->trb_Slices->Size = System::Drawing::Size(172, 32);
			this->trb_Slices->TabIndex = 1;
			this->trb_Slices->TickStyle = System::Windows::Forms::TickStyle::None;
			this->trb_Slices->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// pan_Slices
			// 
			this->pan_Slices->AutoScroll = true;
			this->pan_Slices->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->pan_Slices->Controls->Add(this->openGLPanel);
			this->pan_Slices->Location = System::Drawing::Point(8, 72);
			this->pan_Slices->Name = S"pan_Slices";
			this->pan_Slices->Size = System::Drawing::Size(600, 600);
			this->pan_Slices->TabIndex = 1;
			// 
			// openGLPanel
			// 
			this->openGLPanel->Code = 0;
			this->openGLPanel->Location = System::Drawing::Point(96, 104);
			this->openGLPanel->Name = S"openGLPanel";
			this->openGLPanel->Size = System::Drawing::Size(400, 400);
			this->openGLPanel->TabIndex = 0;
			// 
			// colDia_SliceColor
			// 
			this->colDia_SliceColor->Color = System::Drawing::Color::White;
			this->colDia_SliceColor->FullOpen = true;
			// 
			// SliceViewer
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(616, 678);
			this->Controls->Add(this->pan_Slices);
			this->Controls->Add(this->grb_SliceControl);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->Name = S"SliceViewer";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = S"SliceViewer";
			this->Closing += new System::ComponentModel::CancelEventHandler(this, SliceViewer_Closing);
			this->grb_SliceControl->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_Slices))->EndInit();
			this->pan_Slices->ResumeLayout(false);
			this->ResumeLayout(false);

		}		




	/**
	Eventhandler for window closing events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void SliceViewer_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
	{
		this->m_SubItem_SliceViewer->Checked = false;
		this->m_TbSubItem_SliceViewer->Checked = false;
	}

	/**
	Eventhandler for combobax selected index changed events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionComboBoxSelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
	{
		if(__try_cast<ComboBox*>(sender)->Equals(this->cob_SliceXYZ))
		{
			resizeCanvas();
			drawSlice();

			return;
		}

		if(__try_cast<ComboBox*>(sender)->Equals(this->cob_SliceSize))
		{
			switch(this->cob_SliceSize->get_SelectedIndex())
			{
				case 0: this->m_SamplingFactor = 1; break;
				case 1: this->m_SamplingFactor = 2; break;
				case 2: this->m_SamplingFactor = 3; break;
				case 3: this->m_SamplingFactor = 4; break;
				case 4: this->m_SamplingFactor = 5; break;
				case 5: this->m_SamplingFactor = 6; break;
				case 6: this->m_SamplingFactor = 7; break;
				case 7: this->m_SamplingFactor = 8; break;
			}

			int tempIndex = this->trb_Slices->get_Value();
			resizeCanvas();
			this->trb_Slices->set_Value(tempIndex);

			drawSlice();

			return;
		}
	}

	/**
	Eventhandler for button click events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
	{
		if(__try_cast<Button*>(sender)->Equals(this->but_SliceColor))
		{
			this->colDia_SliceColor->ShowDialog();
			this->pan_SliceColor->set_BackColor(this->colDia_SliceColor->get_Color());
			drawSlice();
		}
	}

	/**
	Eventhandler for trackbar scroll events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionTrackBarScroll(System::Object *  sender, System::EventArgs *  e)
	{
		if(__try_cast<TrackBar*>(sender)->Equals(this->trb_Slices))
		{
			drawSlice();
		}
	}

	/**
	Eventhandler for textbox keydown events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionTextboxKeydown(System::Object *  sender, System::Windows::Forms::KeyEventArgs *  e)
	{
		if(e->get_KeyValue() == 13)
		{
			if(__try_cast<TextBox*>(sender)->Equals(tex_SliceNumber))
			{
				try
				{
					int value = Convert::ToInt32(tex_SliceNumber->get_Text());	
					if(value > this->trb_Slices->get_Maximum())
					{
						value = this->trb_Slices->get_Maximum();
					}
					if(value < this->trb_Slices->get_Minimum())
					{
						value = this->trb_Slices->get_Minimum();
					}
					this->trb_Slices->set_Value(value);
					this->tex_SliceNumber->set_Text(Convert::ToString(value));
				}
				catch(Exception*)
				{
					this->trb_Slices->set_Value(0);
					this->tex_SliceNumber->set_Text(Convert::ToString(0));
				}

				drawSlice();
			}
		}
	}
};
}