#pragma once

#include "IClassificationModel.h"
#include "Transferfunction.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ClassificationModels {

using namespace IDVR::Volume::DataStructures;
using namespace IDVR::Interfaces;

/**
<B>Class for the transfer function classification model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels

Baseclass: IDVR::Interfaces::IClassificationModel

This class calculates the opacity value of a given raysample according to the specified transfer function.

@see LinearClassificationModelSW - the linear classification model.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class TFClassificationModelSW : public IClassificationModel
{

private:

	Transferfunction *m_TransferFunc; /**< The transfer function */

public:

	/**
	Constructor.
	@param n_TransferFunc the transfer function.
	*/
	TFClassificationModelSW(Transferfunction *n_TransferFunc);

	
	/**
	Calculates the opacity vale for a given raysample according to the specified transfer function.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doClassification(RaySample* n_RaySample);

	
	/**
	Sets the transfer function.
	@param n_TransferFunc the transfer function.
	*/
	__property void set_TransferFunc(Transferfunction* n_TransferFunc);
	
	/**
	Returns the transfer function.
	@return The transfer function.
	*/
	__property Transferfunction* get_TransferFunc(void);
};
}
}
}
}