#include "StdAfx.h"
#include ".\ToneShadingModelSW.h"
#using <mscorlib.dll>

#include "DataLoader.h"

using namespace IDVR::Volume;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

//Standard constructor.
ToneShadingModelSW::ToneShadingModelSW(void)
{
	this->m_DensityScope = DataLoader::DensityScope;
	this->m_GradientMagnitudeThreshold = DataLoader::Default_ToneMagnitudeThreshold;
	this->m_KWarm = DataLoader::Default_ToneKCold;
	this->m_KCold = DataLoader::Default_ToneKWarm;
	this->m_LightDirection = NULL;

	this->m_WarmColor = new float __gc[3];
	this->m_WarmColor[RED] = (float)DataLoader::Default_ToneWarmColor.get_R() / 255.0f;
	this->m_WarmColor[GREEN] = (float)DataLoader::Default_ToneWarmColor.get_G() / 255.0f;
	this->m_WarmColor[BLUE] = (float)DataLoader::Default_ToneWarmColor.get_B() / 255.0f;

	this->m_ColdColor = new float __gc[3];
	this->m_WarmColor[RED] = (float)DataLoader::Default_ToneColdColor.get_R() / 255.0f;
	this->m_WarmColor[GREEN] = (float)DataLoader::Default_ToneColdColor.get_G() / 255.0f;
	this->m_WarmColor[BLUE] = (float)DataLoader::Default_ToneColdColor.get_B() / 255.0f;

	this->m_TransferFunc = Transferfunction::CreateGeneralTransfer(this->m_DensityScope);
	this->m_KTF = DataLoader::Default_ToneKTF;
}

//Constructor.
ToneShadingModelSW::ToneShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_KWarm, Color n_CWarm, double n_KCold, Color n_CCold, Vector3* n_LightDirection)
{
	this->m_DensityScope = n_DensityScope;
	this->m_GradientMagnitudeThreshold = n_GradientMagnitudeThreshold;
	this->m_KWarm = n_KWarm;
	this->m_KCold = n_KCold;
	this->m_LightDirection = n_LightDirection;
	
	this->m_WarmColor = new float __gc[3];
	this->m_WarmColor[RED] = (float)n_CWarm.get_R() / 255.0f;
	this->m_WarmColor[GREEN] = (float)n_CWarm.get_G() / 255.0f;
	this->m_WarmColor[BLUE] = (float)n_CWarm.get_B() / 255.0f;

	this->m_ColdColor = new float __gc[3];
	this->m_ColdColor[RED] = (float)n_CCold.get_R() / 255.0f;
	this->m_ColdColor[GREEN] = (float)n_CCold.get_G() / 255.0f;
	this->m_ColdColor[BLUE] = (float)n_CCold.get_B() / 255.0f;

	this->m_TransferFunc = NULL;
	this->m_KTF = NULL;
}

//Constructor.
ToneShadingModelSW::ToneShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_KWarm, Color n_CWarm, double n_KCold, Color n_CCold, Vector3* n_LightDirection, float n_KTF, Transferfunction* n_TransferFunc)
{
	this->m_DensityScope = n_DensityScope;
	this->m_GradientMagnitudeThreshold = n_GradientMagnitudeThreshold;
	this->m_KWarm = n_KWarm;
	this->m_KCold = n_KCold;
	this->m_LightDirection = n_LightDirection;
	
	this->m_WarmColor = new float __gc[3];
	this->m_WarmColor[RED] = (float)n_CWarm.get_R() / 255.0f;
	this->m_WarmColor[GREEN] = (float)n_CWarm.get_G() / 255.0f;
	this->m_WarmColor[BLUE] = (float)n_CWarm.get_B() / 255.0f;

	this->m_ColdColor = new float __gc[3];
	this->m_ColdColor[RED] = (float)n_CCold.get_R() / 255.0f;
	this->m_ColdColor[GREEN] = (float)n_CCold.get_G() / 255.0f;
	this->m_ColdColor[BLUE] = (float)n_CCold.get_B() / 255.0f;

	this->m_TransferFunc = n_TransferFunc;
	this->m_KTF = n_KTF;
}

//Calculates the Color value for a given raysample.
RaySample* ToneShadingModelSW::doShading(RaySample* n_RaySample)
{
	float temp_Color __gc[] = new float __gc[3];

	//double temp_Intensiity;
	float innerProd = (float)n_RaySample->get_Gradient()->get_GradientVector()->dotProd(this->m_LightDirection);

	if(innerProd > 0.0f && n_RaySample->get_Gradient()->get_ScaledLength() >= this->m_GradientMagnitudeThreshold)
	{
		temp_Color[RED] =   ((1.0f + innerProd) / 2.0f) * (float)this->m_KWarm * this->m_WarmColor[RED] +
									(1.0f - (1.0f + innerProd) / 2.0f) * (float)this->m_KCold * this->m_ColdColor[RED];
		temp_Color[GREEN] = ((1.0f + innerProd) / 2.0f) * (float)this->m_KWarm * this->m_WarmColor[GREEN] +
									(1.0f - (1.0f + innerProd) / 2.0f) * (float)this->m_KCold * this->m_ColdColor[GREEN];
		temp_Color[BLUE] =  ((1.0f + innerProd) / 2.0f) * (float)this->m_KWarm * this->m_WarmColor[BLUE] +
									(1.0f - (1.0f + innerProd) / 2.0f) * (float)this->m_KCold * this->m_ColdColor[BLUE];
	}
	else
		temp_Color[RED] = temp_Color[GREEN] = temp_Color[BLUE] = 0;

	//no transfer funtion is specified => LMip - compositing
	if(this->m_TransferFunc != NULL)
	{
		/*
		temp_Color[RED] = (temp_Color[RED] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[RED])) / 2;
		temp_Color[GREEN] = (temp_Color[GREEN] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[GREEN])) / 2;
		temp_Color[BLUE] = (temp_Color[BLUE] + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[BLUE])) / 2;
		*/

		//blends according to: tone color : transfer function color = 1 - ktf : ktf
		temp_Color[RED] = ((1 - this->m_KTF) * temp_Color[RED]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[RED]);
		temp_Color[GREEN] = ((1 - this->m_KTF) * temp_Color[GREEN]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[GREEN]);
		temp_Color[BLUE] = ((1 - this->m_KTF) * temp_Color[BLUE]) + (this->m_KTF * this->m_TransferFunc->getColor(n_RaySample->get_Density())[BLUE]);
		
	}

	n_RaySample->set_Color(temp_Color);

	return n_RaySample;
}

//Sets the current light direction.
void ToneShadingModelSW::set_LightDirection(Vector3* n_Light)
{
	this->m_LightDirection = n_Light;
}

//Returns the gradient magnitude threshold.
double ToneShadingModelSW::get_ToneGradientMagnitudeThreshold(void)
{
	return this->m_GradientMagnitudeThreshold;
}

//Returns the ratio of the warm color.
double ToneShadingModelSW::get_ToneKWarm(void)
{
	return this->m_KWarm;
}

//Returns the ratio of the cold color.
double ToneShadingModelSW::get_ToneKCold(void)
{
	return this->m_KCold;
}

//Returns the the warm color as Color object.
Color ToneShadingModelSW::get_ToneWarmColor(void)
{
	return Color::FromArgb(Convert::ToInt32(Math::Round(this->m_WarmColor[RED] * 255.0)), 
		                   Convert::ToInt32(Math::Round(this->m_WarmColor[GREEN] * 255.0)), 
						   Convert::ToInt32(Math::Round(this->m_WarmColor[BLUE] * 255.0)));
}

//Returns the the cold color as Color object.
Color ToneShadingModelSW::get_ToneColdColor(void)
{
	return Color::FromArgb(Convert::ToInt32(Math::Round(this->m_ColdColor[RED] * 255.0)), 
		                   Convert::ToInt32(Math::Round(this->m_ColdColor[GREEN] * 255.0)), 
						   Convert::ToInt32(Math::Round(this->m_ColdColor[BLUE] * 255.0)));
}

//Returns the ktf value.
float ToneShadingModelSW::get_ToneKTF(void)
{
	return this->m_KTF;
}