#include "StdAfx.h"
#include ".\Vector3.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;
	
/*
This is the standard constructor.
This standard constructor initialzed all three elements with zero.
*/
Vector3::Vector3(void)
{
	this->m_X = this->m_Y = this->m_Z = 0;
}
/*
Constructor initialization by three parameters.
This constructor initialized all three element of the vector with the specified parameters.
*/
Vector3::Vector3(double n_X, double n_Y, double n_Z)
{
	this->m_X = n_X;
	this->m_Y = n_Y;
	this->m_Z = n_Z;
}
/*
Constructor initialization by an array parameters.
This constructor initialized all three element of the vector with the specified array parameter.
*/
Vector3::Vector3(double n_Value __gc[])
{	
	this->m_X = n_Value[0]; 
	this->m_Y = n_Value[1]; 
	this->m_Z = n_Value[2]; 
}
/*
Constructor initialization by an already existing %Vector3 object.
This constructor initialized all three element of the vector with the values of the specified %Vector3 object.
*/
Vector3::Vector3(Vector3 *n_Vector3)
{
	this->m_X = n_Vector3->X;
	this->m_Y = n_Vector3->Y;
	this->m_Z = n_Vector3->Z;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/
	
//Returns the x element of the specific vector.
double Vector3::get_X(void)
{
	return this->m_X;
}

//Returns the y element of the specific vector.
double Vector3::get_Y(void)
{
	return this->m_Y;
}

//Returns the z element of the specific vector.
double Vector3::get_Z(void)
{
	return this->m_Z;
}

//Sets the x element of the existing vector.
void Vector3::set_X(double n_X)
{
	this->m_X = n_X;
}

//Sets the y element of the existing vector.
void Vector3::set_Y(double n_Y)
{
	this->m_Y = n_Y;
}

//Sets the z element of the existing vector.
void Vector3::set_Z(double n_Z)
{
	this->m_Z = n_Z;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//This function returns the dot product (inner product) of the current %Vector3 object and the specified vector parameter.
double Vector3::dotProd(Vector3* v)
{
	return(this->m_X * v->X + this->m_Y * v->Y + this->m_Z * v->Z);
}

//This function returns the skalar product as a new %Vector3 object of the current %Vector3 object and the specified skalar parameter.
Vector3* Vector3::multSkalar(double d)
{
	Vector3* v_Result = new Vector3();

	v_Result->X = this->m_X * d;
	v_Result->Y = this->m_Y * d;
	v_Result->Z = this->m_Z * d;

	return v_Result;
}

//Returns the magnitude of the specific vector.
double Vector3::magnitude(void)
{
	return (double)Math::Sqrt((this->m_X * this->m_X) + (this->m_Y * this->m_Y) + (this->m_Z * this->m_Z));
}

//This function normalizes the current vector.
void Vector3::normalize(void)
{
	double d_mag = magnitude();
	
	if (d_mag != 0)
	{
		this->m_X /= d_mag;
		this->m_Y /= d_mag;
		this->m_Z /= d_mag;	
	}
}

/*
This function returns the specific element of the current vector. 
This element is specified by the specified parameter whereas the following index enumeration is valid:
	0 means the x elements
	1 means the y elements
	2 means the z elements
*/
double Vector3::getItem(int index)
{
	if (index == 0) return this->m_X;
	else if (index == 1) return this->m_Y;
	else if (index == 2) return this->m_Z;

	return NULL;
}

/*
This function returns the negative vector of the current %Vector3 object. 
This means that the current vector is multiplied with -1 which is actually a skalar product.
*/
Vector3* Vector3::getNegativeVector(void)
{
	return new Vector3(-this->m_X, -this->m_Y, -this->m_Z);
}

//This function returns the all vector elements as an three element float array.
float Vector3::getFloatArrayOfVector(void) __gc[]
{
	float tempValues __gc[] = {(float)this->m_X, (float)this->m_Y, (float)this->m_Z};
	return tempValues;
}

//This function returns the all vector elements as an three element double array.
double Vector3::getDoubleArrayOfVector(void) __gc[]
{
	double tempValues __gc[] = {this->m_X, this->m_Y, this->m_Z};
	return tempValues;
}

/*
This function converts the current vector to an OpenGL compatible vector form.
For correct drawing in OpenGL the x and y elements of an existing %Vector3 object must
be inverted, only after this invertion a correct drawing in OpenGL is guaranteed (is basically a
reflextion about the y and z axis).
*/
Vector3* Vector3::convertToOpenGLAlignedVector(void)
{
	return new Vector3(-this->m_X, -this->m_Y, this->m_Z);
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//This static function created a normalized vector of the specified vector parameter.
Vector3* Vector3::CreateNormalizedVector(Vector3* v)
{
	 double d_mag = v->magnitude();
	
	 if (d_mag != 0)
	 {
		 v->X /= d_mag;
		 v->Y /= d_mag;
		 v->Z /= d_mag;
	 }
	 return v;
}

//This static function returns the sum vector of the two specified vector parameters.
Vector3* Vector3::Addition(Vector3* v1, Vector3* v2)
{
	return new Vector3(v1->X + v2->X, v1->Y + v2->Y, v1->Z + v2->Z);
}

//This static function returns the subtraction vector of the two specified vector parameters.
Vector3* Vector3::Subtraction(Vector3* v1, Vector3* v2)
{
	return new Vector3(v1->X - v2->X, v1->Y - v2->Y, v1->Z - v2->Z); 
}

//This static function returns the cross product (outer product) vector of the two specified vector parameters.
Vector3* Vector3::CrossProd(Vector3 *v1, Vector3 *v2)
{
	Vector3 *temp = new Vector3();
	temp->X = (v1->Y * v2->Z - v1->Z * v2->Y);
	temp->Y = (v1->Z * v2->X - v1->X * v2->Z);
	temp->Z = (v1->X * v2->Y - v1->Y * v2->X);
	return temp;
}

//This static function returns the dot product (inner product) of the two specified vector parameters.
double Vector3::DotProd(Vector3* v1, Vector3* v2)
{
	return v1->dotProd(v2);
}