#include "StdAfx.h"
#include ".\Vector4.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;

/*
This is the standard constructor.
This standard constructor initialized all four elements with zero.
*/
Vector4::Vector4(void)
{
	this->m_X = this->m_Y = this->m_Z = this->m_W = 0.0;
}

/*
Constructor initialization by three parameters.
This constructor initialized all four element of the vector with the specified parameters.
*/
Vector4::Vector4(double n_X, double n_Y, double n_Z, double n_W)
{
	this->m_X = n_X;
	this->m_Y = n_Y;
	this->m_Z = n_Z;
	this->m_W = n_W;
}

/*
Constructor initialization by an array parameters.
This constructor initialized all four element of the vector with the specified array parameter.
*/
Vector4::Vector4(double n_Value __gc[])
{	this->m_X = n_Value[0]; 
	this->m_Y = n_Value[1]; 
	this->m_Z = n_Value[2]; 
	this->m_W = n_Value[3];
}

/*
Constructor initialization by an already existing Vector4 object.
This constructor initialized all three element of the vector with the values of the specified Vector4 object.
*/
Vector4::Vector4(Vector3 *n_Vector3, double n_W)
{
	this->m_X = n_Vector3->X;
	this->m_Y = n_Vector3->Y;
	this->m_Z = n_Vector3->Z;
	this->m_W = n_W;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Returns the x element of the specific vector.
double Vector4::get_X(void)
{
	return this->m_X;
}

//Returns the y element of the specific vector.
double Vector4::get_Y(void)
{
	return this->m_Y;
}

//Returns the z element of the specific vector.
double Vector4::get_Z(void)
{
	return this->m_Z;
}

//Returns the w element of the specific vector.
double Vector4::get_W(void)
{
	return this->m_W;
}

//Sets the x element of the existing vector.
void Vector4::set_X(double n_X)
{
	this->m_X = n_X;
}

//Sets the y element of the existing vector.
void Vector4::set_Y(double n_Y)
{
	this->m_Y = n_Y;
}

//Sets the z element of the existing vector.
void Vector4::set_Z(double n_Z)
{
	this->m_Z = n_Z;
}

//Sets the w element of the existing vector.
void Vector4::set_W(double n_W)
{
	this->m_W = n_W;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//This function returns the dot product (inner product) of the current Vector4 object and the specified vector parameter.
double Vector4::dotProd(Vector4 *v)
{
	return(this->m_X * v->X + this->m_Y * v->Y + this->m_Z * v->Z + this->m_W * v->W);
}

//This function returns the skalar product as a new Vector4 object of the current Vector4 object and the specified skalar parameter.
Vector4* Vector4::multSkalar(double d)
{
	Vector4 *v_Result = new Vector4();

	v_Result->X = this->m_X * d;
	v_Result->Y = this->m_Y * d;
	v_Result->Z = this->m_Z * d;
	v_Result->W = this->m_W * d;

	return v_Result;
}

//Returns the magnitude of the specific vector.
double Vector4::magnitude(void)
{
	return (double)Math::Sqrt((this->m_X * this->m_X) + (this->m_Y * this->m_Y) + (this->m_Z * this->m_Z) + (this->m_W * this->m_W));
}

//This function normalizes the current vector.
void Vector4::normalize(void)
{
	double d_mag = magnitude();
	
	if (d_mag != 0)
	{
		this->m_X /= d_mag;
		this->m_Y /= d_mag;
		this->m_Z /= d_mag;	
		this->m_W /= d_mag;	
	}
}

/*
This function returns the specific element of the current vector. This element is specified by the specified parameter whereas
the following index enumeration is valid:
	0 means the x elements
	1 means the y elements
	2 means the z elements
	3 means the w elements
*/
double Vector4::getItem(int index)
{
	if (index == 0) return this->m_X;
	else if (index == 1) return this->m_Y;
	else if (index == 2) return this->m_Z;
	else if (index == 3) return this->m_W;

	return NULL;
}

/*
This function returns the negative vector of the current %Vector4 object. This means that the current vector is multiplied 
with -1 which is actually a skalar product.
*/
Vector4* Vector4::getNegativeVector(void)
{
	return new Vector4(-this->m_X, -this->m_Y, -this->m_Z, -this->m_W);
}

//This function returns the all vector elements as an three element float array.
float Vector4::getFloatArrayOfVector(void) __gc[]
{
	float tempValues __gc[] = {(float)this->m_X, (float)this->m_Y, (float)this->m_Z, (float)this->m_W};
	return tempValues;
}

//This function returns the all vector elements as an three element double array.
double Vector4::getDoubleArrayOfVector(void) __gc[]
{
	double tempValues __gc[] = {this->m_X, this->m_Y, this->m_Z, this->m_W};
	return tempValues;
}

/*
	This function returns a new instance of the Vector3 class which contains the firest three elements of the
	current %Vector4 object divided by the w element. This means that the x,y and z element will be divided by the w element 
	and afterwards copied to the new %Vector3 instance.
*/
Vector3* Vector4::getVector3(void)
{
	return new Vector3(this->m_X / this->m_W, this->m_Y / this->m_W, this->m_Z / this->m_W);
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//This static function created a normalized vector of the specified vector parameter.
Vector4* Vector4::CreateNormalizedVector(Vector4 *v)
{
	 double d_mag = v->magnitude();
	
	 if (d_mag != 0)
	 {
		 v->X /= d_mag;
		 v->Y /= d_mag;
		 v->Z /= d_mag;
		 v->W /= d_mag;	
	 }
	 return v;
}

//This static function returns the sum vector of the two specified vector parameters.
Vector4* Vector4::Addition(Vector4 *v1, Vector4 *v2)
{
	return new Vector4(v1->X + v2->X, v1->Y + v2->Y, v1->Z + v2->Z, v1->W + v2->W);
}

//This static function returns the subtraction vector of the two specified vector parameters.
Vector4* Vector4::Subtraction(Vector4 *v1, Vector4 *v2)
{
	return new Vector4(v1->X - v2->X, v1->Y - v2->Y, v1->Z - v2->Z, v1->W - v2->W); 
}

//This static function returns the cross product (outer product) vector of the two specified vector parameters.
Vector4* Vector4::CrossProd(Vector4 *v1, Vector4 *v2)
{
	Vector4 *temp =  new Vector4();
	temp->X = v1->Y * v2->Z - v1->Z * v2->Y;
	temp->Y = v1->Z * v2->X - v1->X * v2->Z;
	temp->Z = v1->X * v2->Y - v1->Y * v2->X;
	return temp;
}

//This static function returns the dot product (inner product) of the two specified vector parameters.
double Vector4::DotProd(Vector4 *v1, Vector4 *v2)
{
	return v1->dotProd(v2);
}

/*
This static function returns a new instance of the Vector3 class which contains the firest three elements of the
current Vector4 object divided by the w element. This means that the x,y and z element will be divided by the w element 
and afterwards copied to the new Vector3 instance.
*/
Vector3* Vector4::GetVector3(Vector4 *n_Vector4)
{
	return n_Vector4->getVector3();
}

/*
This static function initalizied a new Vector4 object by the specified Vector3 instance.
Hereby should be noted that the fourth element will be invariably set to zero.
*/
Vector4* Vector4::GetVector4OfVector3(Vector3 *n_Vector3)
{
	return new Vector4(n_Vector3, 0.0);
}

